# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import typing as ty

from keystoneauth1 import adapter
import typing_extensions as ty_ext

from openstack import exceptions
from openstack import resource
from openstack import utils


class Flavor(resource.Resource):
    resource_key = 'flavor'
    resources_key = 'flavors'
    base_path = '/flavors'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_delete = True
    allow_list = True
    allow_commit = True

    _query_mapping = resource.QueryParameters(
        "sort_key",
        "sort_dir",
        "is_public",
        min_disk="minDisk",
        min_ram="minRam",
    )

    # extra_specs introduced in 2.61
    _max_microversion = '2.61'

    # Properties
    #: The name of this flavor.
    name = resource.Body('name', alias='original_name')
    #: The name of this flavor when returned by server list/show
    original_name = resource.Body('original_name')
    #: The description of the flavor.
    description = resource.Body('description')
    #: Size of the disk this flavor offers. *Type: int*
    disk = resource.Body('disk', type=int, default=0)
    #: ``True`` if this is a publicly visible flavor. ``False`` if this is
    #: a private image. *Type: bool*
    is_public = resource.Body(
        'os-flavor-access:is_public', type=bool, default=True
    )
    #: The amount of RAM (in MB) this flavor offers. *Type: int*
    ram = resource.Body('ram', type=int, default=0)
    #: The number of virtual CPUs this flavor offers. *Type: int*
    vcpus = resource.Body('vcpus', type=int, default=0)
    #: Size of the swap partitions.
    swap = resource.Body('swap', type=int, default=0)
    #: Size of the ephemeral data disk attached to this server. *Type: int*
    ephemeral = resource.Body('OS-FLV-EXT-DATA:ephemeral', type=int, default=0)
    #: ``True`` if this flavor is disabled, ``False`` if not. *Type: bool*
    is_disabled = resource.Body('OS-FLV-DISABLED:disabled', type=bool)
    #: The bandwidth scaling factor this flavor receives on the network.
    rxtx_factor = resource.Body('rxtx_factor', type=float)
    # TODO(mordred) extra_specs can historically also come from
    #               OS-FLV-WITH-EXT-SPECS:extra_specs. Do we care?
    #: A dictionary of the flavor's extra-specs key-and-value pairs.
    extra_specs = resource.Body('extra_specs', type=dict, default={})

    def __getattribute__(self, name):
        """Return an attribute on this instance

        This is mostly a pass-through except for a specialization on
        the 'id' name, as this can exist under a different name via the
        `alternate_id` argument to resource.Body.
        """
        if name == "id":
            # ID handling in flavor is very tricky. Sometimes we get ID back,
            # sometimes we get only name (but it is same as id), sometimes we
            # get original_name back, but it is still id.
            # To get this handled try sequentially to access it from various
            # places until we find first non-empty value.
            for xname in ["id", "name", "original_name"]:
                if self._body.get(xname):
                    return self._body[xname]
        else:
            return super().__getattribute__(name)

    @classmethod
    def list(
        cls,
        session: adapter.Adapter,
        paginated: bool = True,
        base_path: str | None = '/flavors/detail',
        allow_unknown_params: bool = False,
        *,
        microversion: str | None = None,
        headers: dict[str, str] | None = None,
        max_items: int | None = None,
        **params: ty.Any,
    ) -> ty.Generator[ty_ext.Self, None, None]:
        # Find will invoke list when name was passed. Since we want to return
        # flavor with details (same as direct get) we need to swap default here
        # and list with "/flavors" if no details explicitely requested
        if 'is_public' not in params or params['is_public'] is None:
            # is_public is ternary - None means give all flavors.
            # Force it to string to avoid requests skipping it.
            params['is_public'] = 'None'
        return super().list(
            session,
            paginated=paginated,
            base_path=base_path,
            allow_unknown_params=allow_unknown_params,
            microversion=microversion,
            headers=headers,
            max_items=max_items,
            **params,
        )

    def _action(self, session, body, microversion=None):
        """Preform flavor actions given the message body."""
        url = utils.urljoin(Flavor.base_path, self.id, 'action')
        headers = {'Accept': ''}
        attrs = {}
        if microversion:
            # Do not reset microversion if it is set on a session level
            attrs['microversion'] = microversion
        response = session.post(url, json=body, headers=headers, **attrs)
        exceptions.raise_from_response(response)
        return response

    def add_tenant_access(self, session, tenant):
        """Adds flavor access to a tenant and flavor.

        :param session: The session to use for making this request.
        :param tenant:
        :returns: None
        """
        body = {'addTenantAccess': {'tenant': tenant}}
        self._action(session, body)

    def remove_tenant_access(self, session, tenant):
        """Removes flavor access to a tenant and flavor.

        :param session: The session to use for making this request.
        :param tenant:
        :returns: None
        """
        body = {'removeTenantAccess': {'tenant': tenant}}
        self._action(session, body)

    def get_access(self, session):
        """Lists tenants who have access to a private flavor

        By default, only administrators can manage private flavor access. A
        private flavor has ``is_public`` set to false while a public flavor has
        ``is_public`` set to true.

        :param session: The session to use for making this request.
        :return: List of dicts with flavor_id and tenant_id attributes
        """
        url = utils.urljoin(Flavor.base_path, self.id, 'os-flavor-access')
        response = session.get(url)
        exceptions.raise_from_response(response)
        return response.json().get('flavor_access', [])

    def fetch_extra_specs(self, session):
        """Fetch extra specs of the flavor

        Starting with 2.61 extra specs are returned with the flavor details,
        before that a separate call is required.

        :param session: The session to use for making this request.
        :returns: The updated flavor.
        """
        url = utils.urljoin(Flavor.base_path, self.id, 'os-extra_specs')
        microversion = self._get_microversion(session)
        response = session.get(url, microversion=microversion)
        exceptions.raise_from_response(response)
        specs = response.json().get('extra_specs', {})
        self._update(extra_specs=specs)
        return self

    def create_extra_specs(self, session, specs):
        """Creates extra specs for a flavor.

        :param session: The session to use for making this request.
        :param specs:
        :returns: The updated flavor.
        """
        url = utils.urljoin(Flavor.base_path, self.id, 'os-extra_specs')
        microversion = self._get_microversion(session)
        response = session.post(
            url, json={'extra_specs': specs}, microversion=microversion
        )
        exceptions.raise_from_response(response)
        specs = response.json().get('extra_specs', {})
        self._update(extra_specs=specs)
        return self

    def get_extra_specs_property(self, session, prop):
        """Get an individual extra spec property.

        :param session: The session to use for making this request.
        :param prop: The property to fetch.
        :returns: The value of the property if it exists, else ``None``.
        """
        url = utils.urljoin(Flavor.base_path, self.id, 'os-extra_specs', prop)
        microversion = self._get_microversion(session)
        response = session.get(url, microversion=microversion)
        exceptions.raise_from_response(response)
        val = response.json().get(prop)
        return val

    def update_extra_specs_property(self, session, prop, val):
        """Update an extra spec for a flavor.

        :param session: The session to use for making this request.
        :param prop: The property to update.
        :param val: The value to update with.
        :returns: The updated value of the property.
        """
        url = utils.urljoin(Flavor.base_path, self.id, 'os-extra_specs', prop)
        microversion = self._get_microversion(session)
        response = session.put(
            url, json={prop: val}, microversion=microversion
        )
        exceptions.raise_from_response(response)
        val = response.json().get(prop)
        return val

    def delete_extra_specs_property(self, session, prop):
        """Delete an extra spec for a flavor.

        :param session: The session to use for making this request.
        :param prop: The property to delete.
        :returns: None
        """
        url = utils.urljoin(Flavor.base_path, self.id, 'os-extra_specs', prop)
        microversion = self._get_microversion(session)
        response = session.delete(url, microversion=microversion)
        exceptions.raise_from_response(response)


# TODO(stephenfin): Deprecate this for removal in 2.0
FlavorDetail = Flavor



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.key_manager.v1 import _format
from openstack import resource


class SecretStore(resource.Resource):
    resources_key = 'secret_stores'
    base_path = '/secret-stores'

    # capabilities
    allow_create = False
    allow_fetch = True
    allow_commit = False
    allow_delete = False
    allow_list = True

    _query_mapping = resource.QueryParameters(
        "name",
        "status",
        "global_default",
        "crypto_plugin",
        "secret_store_plugin",
        "created",
        "updated",
    )

    # Properties
    #: The name of the secret store
    name = resource.Body('name')
    #: The status of the secret store
    status = resource.Body('status')
    #: Timestamp of when the secret store was created
    created_at = resource.Body('created')
    #: Timestamp of when the secret store was last updated
    updated_at = resource.Body('updated')
    #: A URI to the secret store
    secret_store_ref = resource.Body('secret_store_ref')
    #: The ID of the secret store
    secret_store_id = resource.Body(
        'secret_store_ref', alternate_id=True, type=_format.HREFToUUID
    )
    #: Flag indicating if this secret store is global default
    global_default = resource.Body('global_default', type=bool)
    #: The crypto plugin name
    crypto_plugin = resource.Body('crypto_plugin')
    #: The secret store plugin name
    secret_store_plugin = resource.Body('secret_store_plugin')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.dns.v2 import blacklist
from openstack.tests.unit import base

IDENTIFIER = '373cb85e-0f4a-487a-846e-dce7a65cca4d'
EXAMPLE = {
    'id': IDENTIFIER,
    'description': 'blacklist test description',
    'pattern': '.*example.com.',
}


class TestBlackList(base.TestCase):
    def test_basic(self):
        sot = blacklist.Blacklist()
        self.assertEqual(None, sot.resource_key)
        self.assertEqual('blacklists', sot.resources_key)
        self.assertEqual('/blacklists', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertEqual('PATCH', sot.commit_method)

    def test_make_it(self):
        sot = blacklist.Blacklist(**EXAMPLE)
        self.assertEqual(IDENTIFIER, sot.id)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['pattern'], sot.pattern)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import exceptions
from openstack import resource
from openstack import utils


class PortBinding(resource.Resource):
    name_attribute = "bindings"
    resource_name = "binding"
    resource_key = 'binding'
    resources_key = 'bindings'
    base_path = '/ports/%(port_id)s/bindings'

    _allow_unknown_attrs_in_body = True

    # capabilities
    allow_create = True
    allow_fetch = False
    allow_commit = True
    allow_delete = True
    allow_list = True

    requires_id = False

    # Properties
    #: The port ID of the binding
    port_id = resource.URI('port_id')
    #: The hostname of the system the agent is running on.
    host = resource.Body('host')
    #: A dictionary that enables the application running on
    #  the specific host to pass and receive vif port information
    #  specific to the networking back-end.
    profile = resource.Body('profile', type=dict)
    #: A dictionary which contains additional information on the
    #  port. The following fields are defined: port_filter and
    #  ovs_hybrid_plug, both are booleans.
    vif_details = resource.Body('vif_details', type=dict)
    #: The type of which mechanism is used for the port.
    #  Currently the following values are supported: ovs, bridge,
    #  macvtap, hw_veb, hostdev_physical, vhostuser, distributed and
    #  other.
    vif_type = resource.Body('vif_type')
    #: The type of vNIC which this port should be attached to.
    #  The valid values are normal, macvtap, direct, baremetal,
    #  direct-physical, virtio-forwarder, smart-nic and remote-managed.
    vnic_type = resource.Body('vnic_type')

    def activate_port_binding(self, session, **attrs):
        host = attrs['host']
        url = utils.urljoin(
            '/ports', self.port_id, 'bindings', host, 'activate'
        )
        resp = session.put(url, json={'binding': attrs})
        exceptions.raise_from_response(resp)
        self._body.attributes.update(resp.json())
        return self

    def delete_port_binding(self, session, host):
        url = utils.urljoin('/ports', self.port_id, 'bindings', host)
        resp = session.delete(url)
        exceptions.raise_from_response(resp)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from openstack.container_infrastructure_management.v1 import cluster
from openstack.tests.unit import base


coe_cluster_obj = dict(
    status="CREATE_IN_PROGRESS",
    cluster_template_id="0562d357-8641-4759-8fed-8173f02c9633",
    uuid="731387cf-a92b-4c36-981e-3271d63e5597",
    links=[{}, {}],
    stack_id="31c1ee6c-081e-4f39-9f0f-f1d87a7defa1",
    keypair="my_keypair",
    master_count=3,
    create_timeout=60,
    node_count=10,
    name="k8s",
    created_at="2016-08-29T06:51:31+00:00",
    api_address="https://172.24.4.6:6443",
    discovery_url="https://discovery.etcd.io/cbeb580da58915809d59ee69348a84f3",
    updated_at="2016-08-29T06:53:24+00:00",
    coe_version="v1.2.0",
    master_addresses=["172.24.4.6"],
    node_addresses=["172.24.4.13"],
    status_reason="Stack CREATE completed successfully",
)


class TestCOEClusters(base.TestCase):
    def _compare_clusters(self, exp, real):
        self.assertDictEqual(
            cluster.Cluster(**exp).to_dict(computed=False),
            real.to_dict(computed=False),
        )

    def get_mock_url(
        self,
        service_type="container-infrastructure-management",
        base_url_append=None,
        append=None,
        resource=None,
    ):
        return super().get_mock_url(
            service_type=service_type,
            resource=resource,
            append=append,
            base_url_append=base_url_append,
        )

    def test_list_coe_clusters(self):
        self.register_uris(
            [
                dict(
                    method="GET",
                    uri=self.get_mock_url(resource="clusters"),
                    json=dict(clusters=[coe_cluster_obj]),
                )
            ]
        )
        cluster_list = self.cloud.list_coe_clusters()
        self._compare_clusters(
            coe_cluster_obj,
            cluster_list[0],
        )
        self.assert_calls()

    def test_create_coe_cluster(self):
        json_response = dict(uuid=coe_cluster_obj.get("uuid"))
        kwargs = dict(
            name=coe_cluster_obj["name"],
            cluster_template_id=coe_cluster_obj["cluster_template_id"],
            master_count=coe_cluster_obj["master_count"],
            node_count=coe_cluster_obj["node_count"],
        )
        self.register_uris(
            [
                dict(
                    method="POST",
                    uri=self.get_mock_url(resource="clusters"),
                    json=json_response,
                    validate=dict(json=kwargs),
                ),
            ]
        )
        response = self.cloud.create_coe_cluster(**kwargs)
        expected = kwargs.copy()
        expected.update(**json_response)
        self._compare_clusters(expected, response)
        self.assert_calls()

    def test_search_coe_cluster_by_name(self):
        self.register_uris(
            [
                dict(
                    method="GET",
                    uri=self.get_mock_url(resource="clusters"),
                    json=dict(clusters=[coe_cluster_obj]),
                )
            ]
        )

        coe_clusters = self.cloud.search_coe_clusters(name_or_id="k8s")

        self.assertEqual(1, len(coe_clusters))
        self.assertEqual(coe_cluster_obj["uuid"], coe_clusters[0]["id"])
        self.assert_calls()

    def test_search_coe_cluster_not_found(self):
        self.register_uris(
            [
                dict(
                    method="GET",
                    uri=self.get_mock_url(resource="clusters"),
                    json=dict(clusters=[coe_cluster_obj]),
                )
            ]
        )

        coe_clusters = self.cloud.search_coe_clusters(
            name_or_id="non-existent"
        )

        self.assertEqual(0, len(coe_clusters))
        self.assert_calls()

    def test_get_coe_cluster(self):
        self.register_uris(
            [
                dict(
                    method="GET",
                    uri=self.get_mock_url(resource="clusters"),
                    json=dict(clusters=[coe_cluster_obj]),
                )
            ]
        )

        r = self.cloud.get_coe_cluster(coe_cluster_obj["name"])
        self.assertIsNotNone(r)
        self._compare_clusters(
            coe_cluster_obj,
            r,
        )
        self.assert_calls()

    def test_get_coe_cluster_not_found(self):
        self.register_uris(
            [
                dict(
                    method="GET",
                    uri=self.get_mock_url(resource="clusters"),
                    json=dict(clusters=[]),
                )
            ]
        )
        r = self.cloud.get_coe_cluster("doesNotExist")
        self.assertIsNone(r)
        self.assert_calls()

    def test_delete_coe_cluster(self):
        self.register_uris(
            [
                dict(
                    method="GET",
                    uri=self.get_mock_url(resource="clusters"),
                    json=dict(clusters=[coe_cluster_obj]),
                ),
                dict(
                    method="DELETE",
                    uri=self.get_mock_url(
                        resource="clusters", append=[coe_cluster_obj['uuid']]
                    ),
                ),
            ]
        )
        self.cloud.delete_coe_cluster(coe_cluster_obj["uuid"])
        self.assert_calls()

    def test_update_coe_cluster(self):
        self.register_uris(
            [
                dict(
                    method="GET",
                    uri=self.get_mock_url(resource="clusters"),
                    json=dict(clusters=[coe_cluster_obj]),
                ),
                dict(
                    method="PATCH",
                    uri=self.get_mock_url(
                        resource="clusters", append=[coe_cluster_obj["uuid"]]
                    ),
                    status_code=200,
                    validate=dict(
                        json=[
                            {
                                "op": "replace",
                                "path": "/node_count",
                                "value": 3,
                            }
                        ]
                    ),
                ),
            ]
        )
        self.cloud.update_coe_cluster(coe_cluster_obj["uuid"], node_count=3)
        self.assert_calls()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.object_store.v1 import account
from openstack.tests.unit import base


CONTAINER_NAME = "mycontainer"

ACCOUNT_EXAMPLE = {
    'content-length': '0',
    'accept-ranges': 'bytes',
    'date': 'Sat, 05 Jul 2014 19:17:40 GMT',
    'x-account-bytes-used': '12345',
    'x-account-container-count': '678',
    'content-type': 'text/plain; charset=utf-8',
    'x-account-object-count': '98765',
    'x-timestamp': '1453413555.88937',
}


class TestAccount(base.TestCase):
    def setUp(self):
        super().setUp()
        self.endpoint = self.cloud.object_store.get_endpoint() + '/'

    def test_basic(self):
        sot = account.Account(**ACCOUNT_EXAMPLE)
        self.assertIsNone(sot.resources_key)
        self.assertIsNone(sot.id)
        self.assertEqual('/', sot.base_path)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_head)
        self.assertTrue(sot.allow_fetch)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_list)
        self.assertFalse(sot.allow_create)

    def test_make_it(self):
        sot = account.Account(**ACCOUNT_EXAMPLE)
        self.assertIsNone(sot.id)
        self.assertEqual(
            int(ACCOUNT_EXAMPLE['x-account-bytes-used']),
            sot.account_bytes_used,
        )
        self.assertEqual(
            int(ACCOUNT_EXAMPLE['x-account-container-count']),
            sot.account_container_count,
        )
        self.assertEqual(
            int(ACCOUNT_EXAMPLE['x-account-object-count']),
            sot.account_object_count,
        )
        self.assertEqual(ACCOUNT_EXAMPLE['x-timestamp'], sot.timestamp)

    def test_set_temp_url_key(self):
        sot = account.Account()
        key = 'super-secure-key'

        self.register_uris(
            [
                dict(
                    method='POST',
                    uri=self.endpoint,
                    status_code=204,
                    validate=dict(
                        headers={'x-account-meta-temp-url-key': key}
                    ),
                ),
                dict(
                    method='HEAD',
                    uri=self.endpoint,
                    headers={'x-account-meta-temp-url-key': key},
                ),
            ]
        )
        sot.set_temp_url_key(self.cloud.object_store, key)
        self.assert_calls()

    def test_set_account_temp_url_key_second(self):
        sot = account.Account()
        key = 'super-secure-key'

        self.register_uris(
            [
                dict(
                    method='POST',
                    uri=self.endpoint,
                    status_code=204,
                    validate=dict(
                        headers={'x-account-meta-temp-url-key-2': key}
                    ),
                ),
                dict(
                    method='HEAD',
                    uri=self.endpoint,
                    headers={'x-account-meta-temp-url-key-2': key},
                ),
            ]
        )
        sot.set_temp_url_key(self.cloud.object_store, key, secondary=True)
        self.assert_calls()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class MetadefResourceType(resource.Resource):
    resources_key = 'resource_types'
    base_path = '/metadefs/resource_types'

    # capabilities
    allow_list = True

    #: The name of metadata definition resource type
    name = resource.Body('name', alternate_id=True)
    #: The date and time when the resource type was created.
    created_at = resource.Body('created_at')
    #: The date and time when the resource type was updated.
    updated_at = resource.Body('updated_at')


class MetadefResourceTypeAssociation(resource.Resource):
    resources_key = 'resource_type_associations'
    base_path = '/metadefs/namespaces/%(namespace_name)s/resource_types'

    # capabilities
    allow_create = True
    allow_delete = True
    allow_list = True

    #: The name of the namespace whose details you want to see.
    namespace_name = resource.URI('namespace_name')
    #: The name of metadata definition resource type
    name = resource.Body('name', alternate_id=True)
    #: The date and time when the resource type was created.
    created_at = resource.Body('created_at')
    #: The date and time when the resource type was updated.
    updated_at = resource.Body('updated_at')
    #: Prefix for any properties in the namespace that you want to apply
    #: to the resource type. If you specify a prefix, you must append
    #: a prefix separator, such as the colon (:) character.
    prefix = resource.Body('prefix')
    #: Some resource types allow more than one key and value pair
    #: for each instance.  For example, the Image service allows
    #: both user and image metadata on volumes. The properties_target parameter
    #: enables a namespace target to remove the ambiguity
    properties_target = resource.Body('properties_target')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.baremetal.v1 import chassis
from openstack.tests.unit import base


FAKE = {
    "created_at": "2016-08-18T22:28:48.165105+00:00",
    "description": "Sample chassis",
    "extra": {},
    "links": [
        {"href": "http://127.0.0.1:6385/v1/chassis/ID", "rel": "self"},
        {"href": "http://127.0.0.1:6385/chassis/ID", "rel": "bookmark"},
    ],
    "nodes": [
        {"href": "http://127.0.0.1:6385/v1/chassis/ID/nodes", "rel": "self"},
        {"href": "http://127.0.0.1:6385/chassis/ID/nodes", "rel": "bookmark"},
    ],
    "updated_at": None,
    "uuid": "dff29d23-1ded-43b4-8ae1-5eebb3e30de1",
}


class TestChassis(base.TestCase):
    def test_basic(self):
        sot = chassis.Chassis()
        self.assertIsNone(sot.resource_key)
        self.assertEqual('chassis', sot.resources_key)
        self.assertEqual('/chassis', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)
        self.assertEqual('PATCH', sot.commit_method)

    def test_instantiate(self):
        sot = chassis.Chassis(**FAKE)
        self.assertEqual(FAKE['uuid'], sot.id)
        self.assertEqual(FAKE['created_at'], sot.created_at)
        self.assertEqual(FAKE['description'], sot.description)
        self.assertEqual(FAKE['extra'], sot.extra)
        self.assertEqual(FAKE['links'], sot.links)
        self.assertEqual(FAKE['nodes'], sot.nodes)
        self.assertEqual(FAKE['updated_at'], sot.updated_at)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import format
from openstack.tests.unit import base


class TestBoolStrFormatter(base.TestCase):
    def test_deserialize(self):
        self.assertTrue(format.BoolStr.deserialize(True))
        self.assertTrue(format.BoolStr.deserialize('True'))
        self.assertTrue(format.BoolStr.deserialize('TRUE'))
        self.assertTrue(format.BoolStr.deserialize('true'))
        self.assertFalse(format.BoolStr.deserialize(False))
        self.assertFalse(format.BoolStr.deserialize('False'))
        self.assertFalse(format.BoolStr.deserialize('FALSE'))
        self.assertFalse(format.BoolStr.deserialize('false'))
        self.assertRaises(ValueError, format.BoolStr.deserialize, None)
        self.assertRaises(ValueError, format.BoolStr.deserialize, '')
        self.assertRaises(ValueError, format.BoolStr.deserialize, 'INVALID')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
fakes
-----

Fakes used for testing
"""

import datetime
import hashlib
import json
import uuid

from openstack.cloud import meta
from openstack.orchestration.util import template_format

PROJECT_ID = '1c36b64c840a42cd9e9b931a369337f0'
FLAVOR_ID = '0c1d9008-f546-4608-9e8f-f8bdaec8dddd'
CHOCOLATE_FLAVOR_ID = '0c1d9008-f546-4608-9e8f-f8bdaec8ddde'
STRAWBERRY_FLAVOR_ID = '0c1d9008-f546-4608-9e8f-f8bdaec8dddf'
COMPUTE_ENDPOINT = 'https://compute.example.com/v2.1'
ORCHESTRATION_ENDPOINT = f'https://orchestration.example.com/v1/{PROJECT_ID}'
NO_MD5 = '93b885adfe0da089cdf634904fd59f71'
NO_SHA256 = '6e340b9cffb37a989ca544e6bb780a2c78901d3fb33738768511a30617afa01d'
FAKE_PUBLIC_KEY = (
    "ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQCkF3MX59OrlBs3dH5CU7lNmvpbrgZxSpyGj"
    "lnE8Flkirnc/Up22lpjznoxqeoTAwTW034k7Dz6aYIrZGmQwe2TkE084yqvlj45Dkyoj95fW/"
    "sZacm0cZNuL69EObEGHdprfGJQajrpz22NQoCD8TFB8Wv+8om9NH9Le6s+WPe98WC77KLw8qg"
    "fQsbIey+JawPWl4O67ZdL5xrypuRjfIPWjgy/VH85IXg/Z/GONZ2nxHgSShMkwqSFECAC5L3P"
    "HB+0+/12M/iikdatFSVGjpuHvkLOs3oe7m6HlOfluSJ85BzLWBbvva93qkGmLg4ZAc8rPh2O+"
    "YIsBUHNLLMM/oQp Generated-by-Nova\n"
)


def make_fake_flavor(flavor_id, name, ram=100, disk=1600, vcpus=24):
    return {
        'OS-FLV-DISABLED:disabled': False,
        'OS-FLV-EXT-DATA:ephemeral': 0,
        'disk': disk,
        'id': flavor_id,
        'links': [
            {
                'href': f'{COMPUTE_ENDPOINT}/flavors/{flavor_id}',
                'rel': 'self',
            },
            {
                'href': f'{COMPUTE_ENDPOINT}/flavors/{flavor_id}',
                'rel': 'bookmark',
            },
        ],
        'name': name,
        'os-flavor-access:is_public': True,
        'ram': ram,
        'rxtx_factor': 1.0,
        'swap': 0,
        'vcpus': vcpus,
    }


FAKE_FLAVOR = make_fake_flavor(FLAVOR_ID, 'vanilla')
FAKE_CHOCOLATE_FLAVOR = make_fake_flavor(
    CHOCOLATE_FLAVOR_ID, 'chocolate', ram=200
)
FAKE_STRAWBERRY_FLAVOR = make_fake_flavor(
    STRAWBERRY_FLAVOR_ID, 'strawberry', ram=300
)
FAKE_FLAVOR_LIST = [FAKE_FLAVOR, FAKE_CHOCOLATE_FLAVOR, FAKE_STRAWBERRY_FLAVOR]
FAKE_TEMPLATE = '''heat_template_version: 2014-10-16

parameters:
  length:
    type: number
    default: 10

resources:
  my_rand:
    type: OS::Heat::RandomString
    properties:
      length: {get_param: length}
outputs:
  rand:
    value:
      get_attr: [my_rand, value]
'''
FAKE_TEMPLATE_CONTENT = template_format.parse(FAKE_TEMPLATE)


def make_fake_server(
    server_id,
    name,
    status='ACTIVE',
    admin_pass=None,
    addresses=None,
    image=None,
    flavor=None,
):
    if addresses is None:
        if status == 'ACTIVE':
            addresses = {
                "private": [
                    {
                        "OS-EXT-IPS-MAC:mac_addr": "fa:16:3e:df:b0:8d",
                        "version": 6,
                        "addr": "fddb:b018:307:0:f816:3eff:fedf:b08d",
                        "OS-EXT-IPS:type": "fixed",
                    },
                    {
                        "OS-EXT-IPS-MAC:mac_addr": "fa:16:3e:df:b0:8d",
                        "version": 4,
                        "addr": "10.1.0.9",
                        "OS-EXT-IPS:type": "fixed",
                    },
                    {
                        "OS-EXT-IPS-MAC:mac_addr": "fa:16:3e:df:b0:8d",
                        "version": 4,
                        "addr": "172.24.5.5",
                        "OS-EXT-IPS:type": "floating",
                    },
                ]
            }
        else:
            addresses = {}
    if image is None:
        image = {"id": "217f3ab1-03e0-4450-bf27-63d52b421e9e", "links": []}
    if flavor is None:
        flavor = {"id": "64", "links": []}

    server = {
        "OS-EXT-STS:task_state": None,
        "addresses": addresses,
        "links": [],
        "image": image,
        "OS-EXT-STS:vm_state": "active",
        "OS-SRV-USG:launched_at": "2017-03-23T23:57:38.000000",
        "flavor": flavor,
        "id": server_id,
        "security_groups": [{"name": "default"}],
        "user_id": "9c119f4beaaa438792ce89387362b3ad",
        "OS-DCF:diskConfig": "MANUAL",
        "accessIPv4": "",
        "accessIPv6": "",
        "progress": 0,
        "OS-EXT-STS:power_state": 1,
        "OS-EXT-AZ:availability_zone": "nova",
        "metadata": {},
        "status": status,
        "updated": "2017-03-23T23:57:39Z",
        "hostId": "89d165f04384e3ffa4b6536669eb49104d30d6ca832bba2684605dbc",
        "OS-SRV-USG:terminated_at": None,
        "key_name": None,
        "name": name,
        "created": "2017-03-23T23:57:12Z",
        "tenant_id": PROJECT_ID,
        "os-extended-volumes:volumes_attached": [],
        "config_drive": "True",
    }
    if admin_pass:
        server['adminPass'] = admin_pass
    return json.loads(json.dumps(server))


def make_fake_keypair(name):
    # Note: this is literally taken from:
    # https://docs.openstack.org/api-ref/compute/
    return {
        "fingerprint": "7e:eb:ab:24:ba:d1:e1:88:ae:9a:fb:66:53:df:d3:bd",
        "name": name,
        "type": "ssh",
        "public_key": FAKE_PUBLIC_KEY,
        "created_at": datetime.datetime.now().isoformat(),
    }


def make_fake_stack(id, name, description=None, status='CREATE_COMPLETE'):
    return {
        'creation_time': '2017-03-23T23:57:12Z',
        'deletion_time': '2017-03-23T23:57:12Z',
        'description': description,
        'id': id,
        'links': [],
        'parent': None,
        'stack_name': name,
        'stack_owner': None,
        'stack_status': status,
        'stack_user_project_id': PROJECT_ID,
        'tags': None,
        'updated_time': '2017-03-23T23:57:12Z',
    }


def make_fake_stack_event(
    id, name, status='CREATE_COMPLETED', resource_name='id'
):
    event_id = uuid.uuid4().hex
    self_url = "{endpoint}/stacks/{name}/{id}/resources/{name}/events/{event}"
    resource_url = "{endpoint}/stacks/{name}/{id}/resources/{name}"
    return {
        "resource_name": id if resource_name == 'id' else name,
        "event_time": "2017-03-26T19:38:18",
        "links": [
            {
                "href": self_url.format(
                    endpoint=ORCHESTRATION_ENDPOINT,
                    name=name,
                    id=id,
                    event=event_id,
                ),
                "rel": "self",
            },
            {
                "href": resource_url.format(
                    endpoint=ORCHESTRATION_ENDPOINT, name=name, id=id
                ),
                "rel": "resource",
            },
            {
                "href": f"{ORCHESTRATION_ENDPOINT}/stacks/{name}/{id}",
                "rel": "stack",
            },
        ],
        "logical_resource_id": name,
        "resource_status": status,
        "resource_status_reason": "",
        "physical_resource_id": id,
        "id": event_id,
    }


def make_fake_image(
    image_id=None,
    md5=NO_MD5,
    sha256=NO_SHA256,
    status='active',
    image_name='fake_image',
    data=None,
    checksum='ee36e35a297980dee1b514de9803ec6d',
):
    if data:
        md5_hash = hashlib.md5(usedforsecurity=False)
        sha256_hash = hashlib.sha256()
        sha512_hash = hashlib.sha512()
        with open(data, 'rb') as file_obj:
            for chunk in iter(lambda: file_obj.read(8192), b''):
                md5_hash.update(chunk)
                sha256_hash.update(chunk)
                sha512_hash.update(chunk)
        md5 = md5_hash.hexdigest()
        sha256 = sha256_hash.hexdigest()
        sha512 = sha512_hash.hexdigest()
    else:
        sha512 = None
    return {
        'image_state': 'available',
        'container_format': 'bare',
        'min_ram': 0,
        'ramdisk_id': 'fake_ramdisk_id',
        'updated_at': '2016-02-10T05:05:02Z',
        'file': '/v2/images/' + image_id + '/file',
        'size': 3402170368,
        'image_type': 'snapshot',
        'disk_format': 'qcow2',
        'id': image_id,
        'schema': '/v2/schemas/image',
        'status': status,
        'tags': [],
        'visibility': 'private',
        'locations': [
            {'url': 'http://127.0.0.1/images/' + image_id, 'metadata': {}}
        ],
        'min_disk': 40,
        'virtual_size': None,
        'name': image_name,
        'checksum': md5 or checksum,
        'created_at': '2016-02-10T05:03:11Z',
        'owner_specified.openstack.md5': md5 or NO_MD5,
        'owner_specified.openstack.sha256': sha256 or NO_SHA256,
        'owner_specified.openstack.object': f'images/{image_name}',
        'protected': False,
        # Add secure hash fields (os_hash_algo and os_hash_value)
        # Default to sha512 if data was provided, otherwise None
        'os_hash_algo': 'sha512' if sha512 else None,
        'os_hash_value': sha512 if sha512 else None,
    }


def make_fake_machine(machine_name, machine_id=None):
    if not machine_id:
        machine_id = uuid.uuid4().hex
    return meta.obj_to_munch(FakeMachine(id=machine_id, name=machine_name))


def make_fake_port(address, node_id=None, port_id=None):
    if not node_id:
        node_id = uuid.uuid4().hex
    if not port_id:
        port_id = uuid.uuid4().hex
    return meta.obj_to_munch(
        FakeMachinePort(id=port_id, address=address, node_id=node_id)
    )


class FakeFloatingIP:
    def __init__(self, id, pool, ip, fixed_ip, instance_id):
        self.id = id
        self.pool = pool
        self.ip = ip
        self.fixed_ip = fixed_ip
        self.instance_id = instance_id


def make_fake_server_group(id, name, policies):
    return json.loads(
        json.dumps(
            {
                'id': id,
                'name': name,
                'policies': policies,
                'members': [],
                'metadata': {},
            }
        )
    )


def make_fake_hypervisor(id, name):
    return json.loads(
        json.dumps(
            {
                'id': id,
                'hypervisor_hostname': name,
                'state': 'up',
                'status': 'enabled',
                "cpu_info": {
                    "arch": "x86_64",
                    "model": "Nehalem",
                    "vendor": "Intel",
                    "features": ["pge", "clflush"],
                    "topology": {"cores": 1, "threads": 1, "sockets": 4},
                },
                "current_workload": 0,
                "disk_available_least": 0,
                "host_ip": "1.1.1.1",
                "free_disk_gb": 1028,
                "free_ram_mb": 7680,
                "hypervisor_type": "fake",
                "hypervisor_version": 1000,
                "local_gb": 1028,
                "local_gb_used": 0,
                "memory_mb": 8192,
                "memory_mb_used": 512,
                "running_vms": 0,
                "service": {"host": "host1", "id": 7, "disabled_reason": None},
                "vcpus": 1,
                "vcpus_used": 0,
            }
        )
    )


class FakeVolume:
    def __init__(self, id, status, name, attachments=[], size=75):
        self.id = id
        self.status = status
        self.name = name
        self.attachments = attachments
        self.size = size
        self.snapshot_id = 'id:snapshot'
        self.description = 'description'
        self.volume_type = 'type:volume'
        self.availability_zone = 'az1'
        self.created_at = '1900-01-01 12:34:56'
        self.updated_at = None
        self.source_volid = '12345'
        self.metadata = {}


class FakeVolumeSnapshot:
    def __init__(self, id, status, name, description, size=75):
        self.id = id
        self.status = status
        self.name = name
        self.description = description
        self.size = size
        self.created_at = '1900-01-01 12:34:56'
        self.updated_at = None
        self.volume_id = '12345'
        self.metadata = {}
        self.is_forced = False


class FakeMachine:
    def __init__(
        self,
        id,
        name=None,
        driver=None,
        driver_info=None,
        chassis_uuid=None,
        instance_info=None,
        instance_uuid=None,
        properties=None,
        reservation=None,
        last_error=None,
        provision_state='available',
    ):
        self.uuid = id
        self.name = name
        self.driver = driver
        self.driver_info = driver_info
        self.chassis_uuid = chassis_uuid
        self.instance_info = instance_info
        self.instance_uuid = instance_uuid
        self.properties = properties
        self.reservation = reservation
        self.last_error = last_error
        self.provision_state = provision_state


class FakeMachinePort:
    def __init__(self, id, address, node_id):
        self.uuid = id
        self.address = address
        self.node_uuid = node_id


def make_fake_neutron_security_group(
    id, name, description, rules, stateful=True, project_id=None
):
    if not rules:
        rules = []
    if not project_id:
        project_id = PROJECT_ID
    return json.loads(
        json.dumps(
            {
                'id': id,
                'name': name,
                'description': description,
                'stateful': stateful,
                'project_id': project_id,
                'tenant_id': project_id,
                'security_group_rules': rules,
            }
        )
    )


def make_fake_nova_security_group_rule(
    id, from_port, to_port, ip_protocol, cidr
):
    return json.loads(
        json.dumps(
            {
                'id': id,
                'from_port': int(from_port),
                'to_port': int(to_port),
                'ip_protcol': 'tcp',
                'ip_range': {'cidr': cidr},
            }
        )
    )


def make_fake_nova_security_group(id, name, description, rules):
    if not rules:
        rules = []
    return json.loads(
        json.dumps(
            {
                'id': id,
                'name': name,
                'description': description,
                'tenant_id': PROJECT_ID,
                'rules': rules,
            }
        )
    )


class FakeNovaSecgroupRule:
    def __init__(
        self,
        id,
        from_port=None,
        to_port=None,
        ip_protocol=None,
        cidr=None,
        parent_group_id=None,
    ):
        self.id = id
        self.from_port = from_port
        self.to_port = to_port
        self.ip_protocol = ip_protocol
        if cidr:
            self.ip_range = {'cidr': cidr}
        self.parent_group_id = parent_group_id


class FakeHypervisor:
    def __init__(self, id, hostname):
        self.id = id
        self.hypervisor_hostname = hostname


class FakeZone:
    def __init__(self, id, name, type_, email, description, ttl, masters):
        self.id = id
        self.name = name
        self.type_ = type_
        self.email = email
        self.description = description
        self.ttl = ttl
        self.masters = masters


class FakeRecordset:
    def __init__(self, zone, id, name, type_, description, ttl, records):
        self.zone = zone
        self.id = id
        self.name = name
        self.type_ = type_
        self.description = description
        self.ttl = ttl
        self.records = records


def make_fake_aggregate(
    id, name, availability_zone='nova', metadata=None, hosts=None
):
    if not metadata:
        metadata = {}
    if not hosts:
        hosts = []
    return json.loads(
        json.dumps(
            {
                "availability_zone": availability_zone,
                "created_at": datetime.datetime.now().isoformat(),
                "deleted": False,
                "deleted_at": None,
                "hosts": hosts,
                "id": int(id),
                "metadata": {
                    "availability_zone": availability_zone,
                },
                "name": name,
                "updated_at": None,
            }
        )
    )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from openstack.message.v2 import _base
from openstack import resource


class Queue(_base.MessageResource):
    resources_key = "queues"
    base_path = "/queues"

    # capabilities
    allow_create = True
    allow_list = True
    allow_fetch = True
    allow_delete = True

    # Properties
    #: The default TTL of messages defined for a queue, which will effect for
    #: any messages posted to the queue.
    default_message_ttl = resource.Body("_default_message_ttl")
    #: Description of the queue.
    description = resource.Body("description")
    #: The max post size of messages defined for a queue, which will effect
    #: for any messages posted to the queue.
    max_messages_post_size = resource.Body("_max_messages_post_size")
    #: Name of the queue. The name is the unique identity of a queue. It
    #: must not exceed 64 bytes in length, and it is limited to US-ASCII
    #: letters, digits, underscores, and hyphens.
    name = resource.Body("name", alternate_id=True)

    def create(self, session, prepend_key=False, base_path=None, **kwargs):
        request = self._prepare_request(
            requires_id=True, prepend_key=prepend_key, base_path=None
        )
        headers = {
            "Client-ID": self.client_id or str(uuid.uuid4()),
            "X-PROJECT-ID": self.project_id or session.get_project_id(),
        }
        request.headers.update(headers)
        response = session.put(
            request.url, json=request.body, headers=request.headers
        )

        self._translate_response(response, has_body=False)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import typing as ty

from openstack.key_manager.v1 import container as _container
from openstack.key_manager.v1 import order as _order
from openstack.key_manager.v1 import project_quota as _project_quota
from openstack.key_manager.v1 import secret as _secret
from openstack.key_manager.v1 import secret_store as _secret_store
from openstack import proxy
from openstack import resource


class Proxy(proxy.Proxy):
    _resource_registry = {
        "container": _container.Container,
        "order": _order.Order,
        "project_quota": _project_quota.ProjectQuota,
        "secret": _secret.Secret,
        "secret_store": _secret_store.SecretStore,
    }

    def create_container(self, **attrs):
        """Create a new container from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~openstack.key_manager.v1.container.Container`,
            comprised of the properties on the Container class.

        :returns: The results of container creation
        :rtype: :class:`~openstack.key_manager.v1.container.Container`
        """
        return self._create(_container.Container, **attrs)

    def delete_container(self, container, ignore_missing=True):
        """Delete a container

        :param container: The value can be either the ID of a container or a
            :class:`~openstack.key_manager.v1.container.Container`
            instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.NotFoundException` will be
            raised when the container does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent container.

        :returns: ``None``
        """
        self._delete(
            _container.Container, container, ignore_missing=ignore_missing
        )

    def find_container(self, name_or_id, ignore_missing=True):
        """Find a single container

        :param name_or_id: The name or ID of a container.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.NotFoundException` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.
        :returns: One :class:`~openstack.key_manager.v1.container.Container`
            or None
        """
        return self._find(
            _container.Container, name_or_id, ignore_missing=ignore_missing
        )

    def get_container(self, container):
        """Get a single container

        :param container: The value can be the ID of a container or a
            :class:`~openstack.key_manager.v1.container.Container`
            instance.

        :returns: One :class:`~openstack.key_manager.v1.container.Container`
        :raises: :class:`~openstack.exceptions.NotFoundException`
            when no resource can be found.
        """
        return self._get(_container.Container, container)

    def containers(self, **query):
        """Return a generator of containers

        :param kwargs query: Optional query parameters to be sent to limit
            the resources being returned.

        :returns: A generator of container objects
        :rtype: :class:`~openstack.key_manager.v1.container.Container`
        """
        return self._list(_container.Container, **query)

    def update_container(self, container, **attrs):
        """Update a container

        :param container: Either the id of a container or a
            :class:`~openstack.key_manager.v1.container.Container` instance.
        :param attrs: The attributes to update on the container represented
            by ``container``.

        :returns: The updated container
        :rtype: :class:`~openstack.key_manager.v1.container.Container`
        """
        return self._update(_container.Container, container, **attrs)

    def create_order(self, **attrs):
        """Create a new order from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~openstack.key_manager.v1.order.Order`,
            comprised of the properties on the Order class.

        :returns: The results of order creation
        :rtype: :class:`~openstack.key_manager.v1.order.Order`
        """
        return self._create(_order.Order, **attrs)

    def delete_order(self, order, ignore_missing=True):
        """Delete an order

        :param order: The value can be either the ID of a order or a
            :class:`~openstack.key_manager.v1.order.Order`
            instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.NotFoundException` will be
            raised when the order does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent order.

        :returns: ``None``
        """
        self._delete(_order.Order, order, ignore_missing=ignore_missing)

    def find_order(self, name_or_id, ignore_missing=True):
        """Find a single order

        :param name_or_id: The name or ID of a order.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.NotFoundException` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.
        :returns: One :class:`~openstack.key_manager.v1.order.Order` or None
        """
        return self._find(
            _order.Order, name_or_id, ignore_missing=ignore_missing
        )

    def get_order(self, order):
        """Get a single order

        :param order: The value can be the ID of an order or a
            :class:`~openstack.key_manager.v1.order.Order`
            instance.

        :returns: One :class:`~openstack.key_manager.v1.order.Order`
        :raises: :class:`~openstack.exceptions.NotFoundException`
            when no resource can be found.
        """
        return self._get(_order.Order, order)

    def orders(self, **query):
        """Return a generator of orders

        :param kwargs query: Optional query parameters to be sent to limit
            the resources being returned.

        :returns: A generator of order objects
        :rtype: :class:`~openstack.key_manager.v1.order.Order`
        """
        return self._list(_order.Order, **query)

    def update_order(self, order, **attrs):
        """Update a order

        :param order: Either the id of a order or a
            :class:`~openstack.key_manager.v1.order.Order` instance.
        :param attrs: The attributes to update on the order represented
            by ``order``.

        :returns: The updated order
        :rtype: :class:`~openstack.key_manager.v1.order.Order`
        """
        return self._update(_order.Order, order, **attrs)

    def create_secret(self, **attrs):
        """Create a new secret from attributes

        :param dict attrs: Keyword arguments which will be used to create a
            :class:`~openstack.key_manager.v1.secret.Secret`,
            comprised of the properties on the Order class.

        :returns: The results of secret creation
        :rtype: :class:`~openstack.key_manager.v1.secret.Secret`
        """
        return self._create(_secret.Secret, **attrs)

    def delete_secret(self, secret, ignore_missing=True):
        """Delete a secret

        :param secret: The value can be either the ID of a secret or a
            :class:`~openstack.key_manager.v1.secret.Secret`
            instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.NotFoundException` will be
            raised when the secret does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent secret.

        :returns: ``None``
        """
        self._delete(_secret.Secret, secret, ignore_missing=ignore_missing)

    def find_secret(self, name_or_id, ignore_missing=True):
        """Find a single secret

        :param name_or_id: The name or ID of a secret.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.NotFoundException` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.
        :returns: One :class:`~openstack.key_manager.v1.secret.Secret` or
            None
        """
        return self._find(
            _secret.Secret, name_or_id, ignore_missing=ignore_missing
        )

    def get_secret(self, secret):
        """Get a single secret

        :param secret: The value can be the ID of a secret or a
            :class:`~openstack.key_manager.v1.secret.Secret`
            instance.

        :returns: One :class:`~openstack.key_manager.v1.secret.Secret`
        :raises: :class:`~openstack.exceptions.NotFoundException`
            when no resource can be found.
        """
        return self._get(_secret.Secret, secret)

    def secrets(self, **query):
        """Return a generator of secrets

        :param kwargs query: Optional query parameters to be sent to limit
            the resources being returned.

        :returns: A generator of secret objects
        :rtype: :class:`~openstack.key_manager.v1.secret.Secret`
        """
        return self._list(_secret.Secret, **query)

    def update_secret(self, secret, **attrs):
        """Update a secret

        :param secret: Either the id of a secret or a
            :class:`~openstack.key_manager.v1.secret.Secret` instance.
        :param attrs: The attributes to update on the secret represented
            by ``secret``.

        :returns: The updated secret
        :rtype: :class:`~openstack.key_manager.v1.secret.Secret`
        """
        return self._update(_secret.Secret, secret, **attrs)

    # ========== Secret Store Operations ==========

    def secret_stores(self, **query):
        """Return a generator of secret stores

        :param kwargs query: Optional query parameters to be sent to limit
            the resources being returned.

        :returns: A generator of secret store objects
        :rtype: :class:`~openstack.key_manager.v1.secret_store.SecretStore`
        """
        return self._list(_secret_store.SecretStore, **query)

    def get_global_default_secret_store(self):
        """Get the global default secret store

        :returns: One
            :class:`~openstack.key_manager.v1.secret_store.SecretStore`
        :raises: :class:`~openstack.exceptions.NotFoundException`
            when no resource can be found.
        """
        return self._get(
            _secret_store.SecretStore,
            None,
            requires_id=False,
            base_path='/secret-stores/global-default',
        )

    def get_preferred_secret_store(self):
        """Get the preferred secret store for the current project

        :returns: One
            :class:`~openstack.key_manager.v1.secret_store.SecretStore`
        :raises: :class:`~openstack.exceptions.NotFoundException`
            when no resource can be found.
        """
        return self._get(
            _secret_store.SecretStore,
            None,
            requires_id=False,
            base_path='/secret-stores/preferred',
        )

    def delete_project_quota(self, project_id, ignore_missing=True):
        """Delete a project quota

        :param project_id: A project ID.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.NotFoundException` will be
            raised when the project quota does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent project quota.

        :returns: ``None``
        """
        self._delete(
            _project_quota.ProjectQuota,
            project_id,
            ignore_missing=ignore_missing,
        )

    def get_project_quota(self, project_id):
        """Get a single project quota

        :param project_id: A project ID.

        :returns: One
            :class:`~openstack.key_manager.v1.project_quota.ProjectQuota`
        :raises: :class:`~openstack.exceptions.NotFoundException`
            when no resource can be found.
        """
        return self._get(_project_quota.ProjectQuota, project_id)

    def update_project_quota(self, project_id, **attrs):
        """Update a project quota

        :param project_id: A project ID.
        :param attrs: The attributes to update on the project quota represented
            by ``project quota``.

        :returns: The updated project quota
        :rtype: :class:`~openstack.key_manager.v1.project_quota.ProjectQuota`
        """
        return self._update(_project_quota.ProjectQuota, project_id, **attrs)

    # ========== Utilities ==========

    def wait_for_status(
        self,
        res: resource.ResourceT,
        status: str,
        failures: list[str] | None = None,
        interval: int | float | None = 2,
        wait: int | None = None,
        attribute: str = 'status',
        callback: ty.Callable[[int], None] | None = None,
    ) -> resource.ResourceT:
        """Wait for the resource to be in a particular status.

        :param session: The session to use for making this request.
        :param resource: The resource to wait on to reach the status. The
            resource must have a status attribute specified via ``attribute``.
        :param status: Desired status of the resource.
        :param failures: Statuses that would indicate the transition
            failed such as 'ERROR'. Defaults to ['ERROR'].
        :param interval: Number of seconds to wait between checks.
        :param wait: Maximum number of seconds to wait for transition.
            Set to ``None`` to wait forever.
        :param attribute: Name of the resource attribute that contains the
            status.
        :param callback: A callback function. This will be called with a single
            value, progress. This is API specific but is generally a percentage
            value from 0-100.

        :return: The updated resource.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` if the
            transition to status failed to occur in ``wait`` seconds.
        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
            transitioned to one of the states in ``failures``.
        :raises: :class:`~AttributeError` if the resource does not have a
            ``status`` attribute
        """
        return resource.wait_for_status(
            self, res, status, failures, interval, wait, attribute, callback
        )

    def wait_for_delete(
        self,
        res: resource.ResourceT,
        interval: int = 2,
        wait: int = 120,
        callback: ty.Callable[[int], None] | None = None,
    ) -> resource.ResourceT:
        """Wait for a resource to be deleted.

        :param res: The resource to wait on to be deleted.
        :param interval: Number of seconds to wait before to consecutive
            checks.
        :param wait: Maximum number of seconds to wait before the change.
        :param callback: A callback function. This will be called with a single
            value, progress, which is a percentage value from 0-100.

        :returns: The resource is returned on success.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
            to delete failed to occur in the specified seconds.
        """
        return resource.wait_for_delete(self, res, interval, wait, callback)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.identity.v3 import endpoint as _endpoint
from openstack.tests.functional import base


class TestEndpoint(base.BaseFunctionalTest):
    def setUp(self):
        super().setUp()

        self.service_name = self.getUniqueString('service')
        self.service_type = self.getUniqueString('type')
        self.service = self.operator_cloud.identity.create_service(
            name=self.service_name,
            type=self.service_type,
        )
        self.addCleanup(
            self.operator_cloud.identity.delete_service, self.service
        )

        self.region_name = self.getUniqueString('region')
        self.region = self.operator_cloud.identity.create_region(
            name=self.region_name
        )
        self.addCleanup(
            self.operator_cloud.identity.delete_region, self.region
        )

        unique_base = self.getUniqueString('endpoint')
        self.test_url = f'https://{unique_base}.example.com/v1'
        self.updated_url = f'https://{unique_base}.example.com/v2'

    def _delete_endpoint(self, endpoint):
        ret = self.operator_cloud.identity.delete_endpoint(endpoint)
        self.assertIsNone(ret)

    def test_endpoint(self):
        # Create public endpoint
        public_endpoint = self.operator_cloud.identity.create_endpoint(
            service_id=self.service.id,
            interface='public',
            url=self.test_url,
            region_id=self.region.id,
            is_enabled=True,
        )
        self.addCleanup(self._delete_endpoint, public_endpoint)
        self.assertIsInstance(public_endpoint, _endpoint.Endpoint)
        self.assertIsNotNone(public_endpoint.id)
        self.assertEqual(self.service.id, public_endpoint.service_id)
        self.assertEqual('public', public_endpoint.interface)
        self.assertEqual(self.test_url, public_endpoint.url)
        self.assertEqual(self.region.id, public_endpoint.region_id)
        self.assertTrue(public_endpoint.is_enabled)

        # Create internal endpoint for filter testing
        internal_endpoint = self.operator_cloud.identity.create_endpoint(
            service_id=self.service.id,
            interface='internal',
            url=self.test_url,
            region_id=self.region.id,
        )
        self.addCleanup(self._delete_endpoint, internal_endpoint)
        self.assertIsInstance(internal_endpoint, _endpoint.Endpoint)
        self.assertIsNotNone(internal_endpoint.id)
        self.assertEqual('internal', internal_endpoint.interface)

        # Update public endpoint
        public_endpoint = self.operator_cloud.identity.update_endpoint(
            public_endpoint,
            url=self.updated_url,
            is_enabled=False,
        )
        self.assertIsInstance(public_endpoint, _endpoint.Endpoint)
        self.assertEqual(self.updated_url, public_endpoint.url)
        self.assertFalse(public_endpoint.is_enabled)

        # Get endpoint by ID
        public_endpoint = self.operator_cloud.identity.get_endpoint(
            public_endpoint.id
        )
        self.assertIsInstance(public_endpoint, _endpoint.Endpoint)
        self.assertEqual(self.updated_url, public_endpoint.url)
        self.assertFalse(public_endpoint.is_enabled)

        # Find endpoint
        found_endpoint = self.operator_cloud.identity.find_endpoint(
            public_endpoint.id
        )
        self.assertIsInstance(found_endpoint, _endpoint.Endpoint)
        self.assertEqual(public_endpoint.id, found_endpoint.id)

        # List endpoints
        endpoints = list(self.operator_cloud.identity.endpoints())
        self.assertIsInstance(endpoints[0], _endpoint.Endpoint)
        endpoint_ids = {ep.id for ep in endpoints}
        self.assertIn(public_endpoint.id, endpoint_ids)
        self.assertIn(internal_endpoint.id, endpoint_ids)

        # Test service filter
        service_endpoints = list(
            self.operator_cloud.identity.endpoints(service_id=self.service.id)
        )
        service_endpoint_ids = {ep.id for ep in service_endpoints}
        self.assertIn(public_endpoint.id, service_endpoint_ids)
        self.assertIn(internal_endpoint.id, service_endpoint_ids)

        # Test interface filter
        public_endpoints = list(
            self.operator_cloud.identity.endpoints(interface='public')
        )
        public_endpoint_ids = {ep.id for ep in public_endpoints}
        self.assertIn(public_endpoint.id, public_endpoint_ids)

        internal_endpoints = list(
            self.operator_cloud.identity.endpoints(interface='internal')
        )
        internal_endpoint_ids = {ep.id for ep in internal_endpoints}
        self.assertIn(internal_endpoint.id, internal_endpoint_ids)

        # Test region filter
        region_endpoints = list(
            self.operator_cloud.identity.endpoints(region_id=self.region.id)
        )
        region_endpoint_ids = {ep.id for ep in region_endpoints}
        self.assertIn(public_endpoint.id, region_endpoint_ids)
        self.assertIn(internal_endpoint.id, region_endpoint_ids)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.accelerator.v2 import accelerator_request as arq
from openstack.tests.unit import base


FAKE_ID = '0725b527-e51a-41df-ad22-adad5f4546ad'
FAKE_RP_UUID = 'f4b7fe6c-8ab4-4914-a113-547af022935b'
FAKE_INSTANCE_UUID = '1ce4a597-9836-4e02-bea1-a3a6cbe7b9f9'
FAKE_ATTACH_INFO_STR = (
    '{"bus": "5e", "device": "00", "domain": "0000", "function": "1"}'
)

FAKE = {
    'uuid': FAKE_ID,
    'device_profile_name': 'fake-devprof',
    'device_profile_group_id': 0,
    'device_rp_uuid': FAKE_RP_UUID,
    'instance_uuid': FAKE_INSTANCE_UUID,
    'attach_handle_type': 'PCI',
    'attach_handle_info': FAKE_ATTACH_INFO_STR,
}


class TestAcceleratorRequest(base.TestCase):
    def test_basic(self):
        sot = arq.AcceleratorRequest()
        self.assertEqual('arq', sot.resource_key)
        self.assertEqual('arqs', sot.resources_key)
        self.assertEqual('/accelerator_requests', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_patch)

    def test_make_it(self):
        sot = arq.AcceleratorRequest(**FAKE)
        self.assertEqual(FAKE_ID, sot.uuid)
        self.assertEqual(FAKE['device_profile_name'], sot.device_profile_name)
        self.assertEqual(
            FAKE['device_profile_group_id'], sot.device_profile_group_id
        )
        self.assertEqual(FAKE_RP_UUID, sot.device_rp_uuid)
        self.assertEqual(FAKE_INSTANCE_UUID, sot.instance_uuid)
        self.assertEqual(FAKE['attach_handle_type'], sot.attach_handle_type)
        self.assertEqual(FAKE_ATTACH_INFO_STR, sot.attach_handle_info)



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
test_cloud_services
----------------------------------

Tests Keystone services commands.
"""

import warnings

from testtools import matchers

from openstack import exceptions
from openstack.tests.unit import base


class CloudServices(base.TestCase):
    def setUp(self, cloud_config_fixture='clouds.yaml'):
        super().setUp(cloud_config_fixture)

    def get_mock_url(
        self,
        service_type='identity',
        interface='public',
        resource='services',
        append=None,
        base_url_append='v3',
        qs_elements=None,
    ):
        return super().get_mock_url(
            service_type,
            interface,
            resource,
            append,
            base_url_append,
            qs_elements,
        )

    def test_create_service_v3(self):
        service_data = self._get_service_data(
            name='a service', type='network', description='A test service'
        )
        self.register_uris(
            [
                dict(
                    method='POST',
                    uri=self.get_mock_url(),
                    status_code=200,
                    json=service_data.json_response_v3,
                    validate=dict(json={'service': service_data.json_request}),
                )
            ]
        )

        service = self.cloud.create_service(
            name=service_data.service_name,
            service_type=service_data.service_type,
            description=service_data.description,
        )
        self.assertThat(
            service.name, matchers.Equals(service_data.service_name)
        )
        self.assertThat(service.id, matchers.Equals(service_data.service_id))
        self.assertThat(
            service.description, matchers.Equals(service_data.description)
        )
        self.assertThat(
            service.type, matchers.Equals(service_data.service_type)
        )
        self.assert_calls()

    def test_update_service_v3(self):
        service_data = self._get_service_data(
            name='a service', type='network', description='A test service'
        )
        request = service_data.json_request.copy()
        request['enabled'] = False
        resp = service_data.json_response_v3.copy()
        resp['enabled'] = False
        request.pop('description')
        request.pop('name')
        request.pop('type')
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(append=[service_data.service_id]),
                    status_code=200,
                    json=service_data.json_request,
                ),
                dict(
                    method='PATCH',
                    uri=self.get_mock_url(append=[service_data.service_id]),
                    status_code=200,
                    json=resp,
                    validate=dict(json={'service': request}),
                ),
            ]
        )

        service = self.cloud.update_service(
            service_data.service_id, enabled=False
        )
        self.assertThat(
            service.name, matchers.Equals(service_data.service_name)
        )
        self.assertThat(service.id, matchers.Equals(service_data.service_id))
        self.assertThat(
            service.description, matchers.Equals(service_data.description)
        )
        self.assertThat(
            service.type, matchers.Equals(service_data.service_type)
        )
        self.assert_calls()

    def test_list_services(self):
        service_data = self._get_service_data()
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(),
                    status_code=200,
                    json={
                        'services': [service_data.json_response_v3['service']]
                    },
                )
            ]
        )
        services = self.cloud.list_services()
        self.assertThat(len(services), matchers.Equals(1))
        self.assertThat(
            services[0].id, matchers.Equals(service_data.service_id)
        )
        self.assertThat(
            services[0].name, matchers.Equals(service_data.service_name)
        )
        self.assertThat(
            services[0].type, matchers.Equals(service_data.service_type)
        )
        self.assert_calls()

    def test_get_service(self):
        service_data = self._get_service_data()
        service2_data = self._get_service_data()

        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(append=[service_data.service_id]),
                    status_code=200,
                    json=service_data.json_response_v3,
                ),
                # you can't retrieve by name
                dict(
                    method='GET',
                    uri=self.get_mock_url(append=[service_data.service_name]),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(),
                    status_code=200,
                    json={
                        'services': [
                            service_data.json_response_v3['service'],
                            service2_data.json_response_v3['service'],
                        ]
                    },
                ),
                # you can't retrieve by name, especially if it doesn't exist
                dict(
                    method='GET',
                    uri=self.get_mock_url(append=['INVALID SERVICE']),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        qs_elements=['name=INVALID SERVICE']
                    ),
                    json={'services': []},
                ),
            ]
        )

        # Search by id
        service = self.cloud.get_service(name_or_id=service_data.service_id)
        self.assertThat(service.id, matchers.Equals(service_data.service_id))

        # Search by name
        service = self.cloud.get_service(name_or_id=service_data.service_name)
        # test we are getting exactly 1 element
        self.assertThat(service.id, matchers.Equals(service_data.service_id))

        # Not found
        service = self.cloud.get_service(name_or_id='INVALID SERVICE')
        self.assertIs(None, service)

    def test_get_service__multiple_matches(self):
        service_a_data = self._get_service_data(type='type2')
        service_b_data = self._get_service_data(type='type2')

        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(),
                    status_code=200,
                    json={
                        'services': [
                            service_a_data.json_response_v3['service'],
                            service_b_data.json_response_v3['service'],
                        ]
                    },
                ),
            ]
        )

        # Multiple matches
        # test we are getting an Exception
        with warnings.catch_warnings(record=True):
            self.assertRaises(
                exceptions.SDKException,
                self.cloud.get_service,
                name_or_id=None,
                filters={'type': 'type2'},
            )
        self.assert_calls()

    def test_search_services(self):
        service_data = self._get_service_data()
        service2_data = self._get_service_data(type=service_data.service_type)
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(),
                    status_code=200,
                    json={
                        'services': [
                            service_data.json_response_v3['service'],
                            service2_data.json_response_v3['service'],
                        ]
                    },
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(),
                    status_code=200,
                    json={
                        'services': [
                            service_data.json_response_v3['service'],
                            service2_data.json_response_v3['service'],
                        ]
                    },
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(),
                    status_code=200,
                    json={
                        'services': [
                            service_data.json_response_v3['service'],
                            service2_data.json_response_v3['service'],
                        ]
                    },
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(),
                    status_code=200,
                    json={
                        'services': [
                            service_data.json_response_v3['service'],
                            service2_data.json_response_v3['service'],
                        ]
                    },
                ),
            ]
        )

        # Search by id
        services = self.cloud.search_services(
            name_or_id=service_data.service_id
        )
        # test we are getting exactly 1 element
        self.assertThat(len(services), matchers.Equals(1))
        self.assertThat(
            services[0].id, matchers.Equals(service_data.service_id)
        )

        # Search by name
        services = self.cloud.search_services(
            name_or_id=service_data.service_name
        )
        # test we are getting exactly 1 element
        self.assertThat(len(services), matchers.Equals(1))
        self.assertThat(
            services[0].name, matchers.Equals(service_data.service_name)
        )

        # Not found
        services = self.cloud.search_services(name_or_id='!INVALID!')
        self.assertThat(len(services), matchers.Equals(0))

        # Multiple matches
        services = self.cloud.search_services(
            filters={'type': service_data.service_type}
        )
        # test we are getting exactly 2 elements
        self.assertThat(len(services), matchers.Equals(2))
        self.assertThat(
            services[0].id, matchers.Equals(service_data.service_id)
        )
        self.assertThat(
            services[1].id, matchers.Equals(service2_data.service_id)
        )
        self.assert_calls()

    def test_delete_service(self):
        service_data = self._get_service_data()
        self.register_uris(
            [
                # you can't retrieve by name
                dict(
                    method='GET',
                    uri=self.get_mock_url(append=[service_data.service_name]),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        qs_elements=[f'name={service_data.service_name}']
                    ),
                    status_code=200,
                    json={
                        'services': [service_data.json_response_v3['service']]
                    },
                ),
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(append=[service_data.service_id]),
                    status_code=204,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(append=[service_data.service_id]),
                    status_code=200,
                    json=service_data.json_response_v3,
                ),
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(append=[service_data.service_id]),
                    status_code=204,
                ),
            ]
        )

        # Delete by name
        self.cloud.delete_service(name_or_id=service_data.service_name)

        # Delete by id
        self.cloud.delete_service(service_data.service_id)

        self.assert_calls()



# Copyright 2019 Rackspace, US Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import exceptions
from openstack import resource
from openstack import utils


class Amphora(resource.Resource):
    resource_key = 'amphora'
    resources_key = 'amphorae'
    base_path = '/octavia/amphorae'

    # capabilities
    allow_create = False
    allow_fetch = True
    allow_commit = False
    allow_delete = False
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'id',
        'loadbalancer_id',
        'compute_id',
        'lb_network_ip',
        'vrrp_ip',
        'ha_ip',
        'vrrp_port_id',
        'ha_port_id',
        'cert_expiration',
        'cert_busy',
        'role',
        'status',
        'vrrp_interface',
        'vrrp_id',
        'vrrp_priority',
        'cached_zone',
        'created_at',
        'updated_at',
        'image_id',
        'compute_flavor',
    )

    # Properties
    #: The ID of the amphora.
    id = resource.Body('id')
    #: The ID of the load balancer.
    loadbalancer_id = resource.Body('loadbalancer_id')
    #: The ID of the amphora resource in the compute system.
    compute_id = resource.Body('compute_id')
    #: The management IP of the amphora.
    lb_network_ip = resource.Body('lb_network_ip')
    #: The address of the vrrp port on the amphora.
    vrrp_ip = resource.Body('vrrp_ip')
    #: The IP address of the Virtual IP (VIP).
    ha_ip = resource.Body('ha_ip')
    #: The vrrp port's ID in the networking system.
    vrrp_port_id = resource.Body('vrrp_port_id')
    #: The ID of the Virtual IP (VIP) port.
    ha_port_id = resource.Body('ha_port_id')
    #: The date the certificate for the amphora expires.
    cert_expiration = resource.Body('cert_expiration')
    #: Whether the certificate is in the process of being replaced.
    cert_busy = resource.Body('cert_busy')
    #: The role configured for the amphora. One of STANDALONE, MASTER, BACKUP.
    role = resource.Body('role')
    #: The status of the amphora. One of: BOOTING, ALLOCATED, READY,
    #: PENDING_CREATE, PENDING_DELETE, DELETED, ERROR.
    status = resource.Body('status')
    #: The bound interface name of the vrrp port on the amphora.
    vrrp_interface = resource.Body('vrrp_interface')
    #: The vrrp group's ID for the amphora.
    vrrp_id = resource.Body('vrrp_id')
    #: The priority of the amphora in the vrrp group.
    vrrp_priority = resource.Body('vrrp_priority')
    #: The availability zone of a compute instance, cached at create time.
    cached_zone = resource.Body('cached_zone')
    #: The UTC date and timestamp when the resource was created.
    created_at = resource.Body('created_at')
    #: The UTC date and timestamp when the resource was last updated.
    updated_at = resource.Body('updated_at')
    #: The ID of the glance image used for the amphora.
    image_id = resource.Body('image_id')
    #: The ID of the compute flavor used for the amphora.
    compute_flavor = resource.Body('compute_flavor')

    def configure(self, session):
        """Configure load balancer.

        Update the amphora agent configuration. This will push the new
        configuration to the amphora agent and will update the configuration
        options that are mutatable.

        :param session: The session to use for making this request.
        :returns: None
        """
        session = self._get_session(session)
        version = self._get_microversion(session)
        request = self._prepare_request(requires_id=True)
        request.url = utils.urljoin(request.url, 'config')

        response = session.put(
            request.url,
            headers=request.headers,
            microversion=version,
        )

        msg = f"Failed to configure load balancer {self.id}"
        exceptions.raise_from_response(response, error_message=msg)

    def failover(self, session):
        """Failover load balancer.

        :param session: The session to use for making this request.
        :returns: None
        """
        session = self._get_session(session)
        version = self._get_microversion(session)
        request = self._prepare_request(requires_id=True)
        request.url = utils.urljoin(request.url, 'failover')

        response = session.put(
            request.url,
            headers=request.headers,
            microversion=version,
        )

        msg = f"Failed to failover load balancer {self.id}"
        exceptions.raise_from_response(response, error_message=msg)


# TODO(stephenfin): Delete this: it's useless
class AmphoraConfig(resource.Resource):
    base_path = '/octavia/amphorae/%(amphora_id)s/config'

    # capabilities
    allow_create = False
    allow_fetch = False
    allow_commit = True
    allow_delete = False
    allow_list = False
    allow_empty_commit = True

    requires_id = False

    # Properties
    #: The ID of the amphora.
    amphora_id = resource.URI('amphora_id')

    # The default _update code path also has no way to pass has_body into this
    # function, so overriding the method here.
    def commit(
        self, session, prepend_key=True, has_body=False, *args, **kwargs
    ):
        return super().commit(session, prepend_key, has_body, *args, *kwargs)


# TODO(stephenfin): Delete this: it's useless
class AmphoraFailover(resource.Resource):
    base_path = '/octavia/amphorae/%(amphora_id)s/failover'

    # capabilities
    allow_create = False
    allow_fetch = False
    allow_commit = True
    allow_delete = False
    allow_list = False
    allow_empty_commit = True

    requires_id = False

    # Properties
    #: The ID of the amphora.
    amphora_id = resource.URI('amphora_id')

    # The default _update code path also has no way to pass has_body into this
    # function, so overriding the method here.
    def commit(
        self, session, prepend_key=True, has_body=False, *args, **kwargs
    ):
        return super().commit(session, prepend_key, has_body, *args, *kwargs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import yaml

from openstack import exceptions
from openstack.orchestration.v1 import stack
from openstack.tests.functional import base
from openstack.tests.functional.network.v2 import test_network


class TestStack(base.BaseFunctionalTest):
    NAME = 'test_stack'
    stack = None
    network = None
    subnet = None
    cidr = '10.99.99.0/16'

    _wait_for_timeout_key = 'OPENSTACKSDK_FUNC_TEST_TIMEOUT_ORCHESTRATION'

    def setUp(self):
        super().setUp()
        self.require_service('orchestration')

        if self.operator_cloud.compute.find_keypair(self.NAME) is None:
            self.operator_cloud.compute.create_keypair(name=self.NAME)
        image = next(self.operator_cloud.image.images())
        tname = "openstack/tests/functional/orchestration/v1/hello_world.yaml"
        with open(tname) as f:
            template = yaml.safe_load(f)
        # TODO(mordred) Fix the need for this. We have better support in
        # the shade layer.
        template['heat_template_version'] = '2013-05-23'
        self.network, self.subnet = test_network.create_network(
            self.operator_cloud, self.NAME, self.cidr
        )
        parameters = {
            'image': image.id,
            'key_name': self.NAME,
            'network': self.network.id,
        }
        sot = self.operator_cloud.orchestration.create_stack(
            name=self.NAME,
            parameters=parameters,
            template=template,
        )
        assert isinstance(sot, stack.Stack)
        self.assertEqual(True, (sot.id is not None))
        self.stack = sot
        self.assertEqual(self.NAME, sot.name)
        self.operator_cloud.orchestration.wait_for_status(
            sot,
            status='CREATE_COMPLETE',
            failures=['CREATE_FAILED'],
            wait=self._wait_for_timeout,
        )

    def tearDown(self):
        self.operator_cloud.orchestration.delete_stack(
            self.stack, ignore_missing=False
        )
        self.operator_cloud.compute.delete_keypair(self.NAME)
        # Need to wait for the stack to go away before network delete
        try:
            self.operator_cloud.orchestration.wait_for_status(
                self.stack, 'DELETE_COMPLETE', wait=self._wait_for_timeout
            )
        except exceptions.NotFoundException:
            pass
        test_network.delete_network(
            self.operator_cloud, self.network, self.subnet
        )
        super().tearDown()

    def test_list(self):
        names = [o.name for o in self.operator_cloud.orchestration.stacks()]
        self.assertIn(self.NAME, names)

    def test_suspend_resume(self):
        # given
        suspend_status = "SUSPEND_COMPLETE"
        resume_status = "RESUME_COMPLETE"

        # when
        self.operator_cloud.orchestration.suspend_stack(self.stack)
        sot = self.operator_cloud.orchestration.wait_for_status(
            self.stack, suspend_status, wait=self._wait_for_timeout
        )

        # then
        self.assertEqual(suspend_status, sot.status)

        # when
        self.operator_cloud.orchestration.resume_stack(self.stack)
        sot = self.operator_cloud.orchestration.wait_for_status(
            self.stack, resume_status, wait=self._wait_for_timeout
        )

        # then
        self.assertEqual(resume_status, sot.status)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.placement.v1 import _proxy
from openstack.placement.v1 import resource_class
from openstack.placement.v1 import resource_provider
from openstack.placement.v1 import resource_provider_inventory
from openstack.tests.unit import test_proxy_base as test_proxy_base


class TestPlacementProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super().setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestPlacementResourceClass(TestPlacementProxy):
    def test_resource_class_create(self):
        self.verify_create(
            self.proxy.create_resource_class,
            resource_class.ResourceClass,
        )

    def test_resource_class_delete(self):
        self.verify_delete(
            self.proxy.delete_resource_class,
            resource_class.ResourceClass,
            False,
        )

    def test_resource_class_update(self):
        self.verify_update(
            self.proxy.update_resource_class,
            resource_class.ResourceClass,
            False,
        )

    def test_resource_class_get(self):
        self.verify_get(
            self.proxy.get_resource_class,
            resource_class.ResourceClass,
        )

    def test_resource_classes(self):
        self.verify_list(
            self.proxy.resource_classes,
            resource_class.ResourceClass,
        )


class TestPlacementResourceProvider(TestPlacementProxy):
    def test_resource_provider_create(self):
        self.verify_create(
            self.proxy.create_resource_provider,
            resource_provider.ResourceProvider,
        )

    def test_resource_provider_delete(self):
        self.verify_delete(
            self.proxy.delete_resource_provider,
            resource_provider.ResourceProvider,
            False,
        )

    def test_resource_provider_update(self):
        self.verify_update(
            self.proxy.update_resource_provider,
            resource_provider.ResourceProvider,
            False,
        )

    def test_resource_provider_get(self):
        self.verify_get(
            self.proxy.get_resource_provider,
            resource_provider.ResourceProvider,
        )

    def test_resource_providers(self):
        self.verify_list(
            self.proxy.resource_providers,
            resource_provider.ResourceProvider,
        )

    def test_resource_provider_set_aggregates(self):
        self._verify(
            'openstack.placement.v1.resource_provider.ResourceProvider.set_aggregates',
            self.proxy.set_resource_provider_aggregates,
            method_args=['value', 'a', 'b'],
            expected_args=[self.proxy],
            expected_kwargs={'aggregates': ('a', 'b')},
        )

    def test_resource_provider_get_aggregates(self):
        self._verify(
            'openstack.placement.v1.resource_provider.ResourceProvider.fetch_aggregates',
            self.proxy.get_resource_provider_aggregates,
            method_args=['value'],
            expected_args=[self.proxy],
        )


class TestPlacementResourceProviderInventory(TestPlacementProxy):
    def test_resource_provider_inventory_create(self):
        self.verify_create(
            self.proxy.create_resource_provider_inventory,
            resource_provider_inventory.ResourceProviderInventory,
            method_kwargs={
                'resource_provider': 'test_id',
                'resource_class': 'CUSTOM_FOO',
                'total': 20,
            },
            expected_kwargs={
                'resource_provider_id': 'test_id',
                'resource_class': 'CUSTOM_FOO',
                'total': 20,
            },
        )

    def test_resource_provider_inventory_delete(self):
        self.verify_delete(
            self.proxy.delete_resource_provider_inventory,
            resource_provider_inventory.ResourceProviderInventory,
            ignore_missing=False,
            method_kwargs={'resource_provider': 'test_id'},
            expected_kwargs={'resource_provider_id': 'test_id'},
        )

    def test_resource_provider_inventory_update(self):
        self.verify_update(
            self.proxy.update_resource_provider_inventory,
            resource_provider_inventory.ResourceProviderInventory,
            method_kwargs={
                'resource_provider': 'test_id',
                'resource_provider_generation': 1,
            },
            expected_kwargs={
                'resource_provider_id': 'test_id',
                'resource_provider_generation': 1,
            },
        )

    def test_resource_provider_inventory_get(self):
        self.verify_get(
            self.proxy.get_resource_provider_inventory,
            resource_provider_inventory.ResourceProviderInventory,
            method_kwargs={'resource_provider': 'test_id'},
            expected_kwargs={'resource_provider_id': 'test_id'},
        )

    def test_resource_provider_inventories(self):
        self.verify_list(
            self.proxy.resource_provider_inventories,
            resource_provider_inventory.ResourceProviderInventory,
            method_kwargs={'resource_provider': 'test_id'},
            expected_kwargs={'resource_provider_id': 'test_id'},
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.shared_file_system.v2 import share_access_rule
from openstack.tests.unit import base

EXAMPLE = {
    "access_level": "rw",
    "state": "error",
    "id": "507bf114-36f2-4f56-8cf4-857985ca87c1",
    "share_id": "fb213952-2352-41b4-ad7b-2c4c69d13eef",
    "access_type": "cert",
    "access_to": "example.com",
    "access_key": None,
    "created_at": "2021-09-12T02:01:04.000000",
    "updated_at": "2021-09-12T02:01:04.000000",
    "metadata": {"key1": "value1", "key2": "value2"},
}


class TestShareAccessRule(base.TestCase):
    def test_basic(self):
        rules_resource = share_access_rule.ShareAccessRule()
        self.assertEqual('access_list', rules_resource.resources_key)
        self.assertEqual('/share-access-rules', rules_resource.base_path)
        self.assertTrue(rules_resource.allow_list)

        self.assertDictEqual(
            {"limit": "limit", "marker": "marker", "share_id": "share_id"},
            rules_resource._query_mapping._mapping,
        )

    def test_make_share_access_rules(self):
        rules_resource = share_access_rule.ShareAccessRule(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], rules_resource.id)
        self.assertEqual(EXAMPLE['access_level'], rules_resource.access_level)
        self.assertEqual(EXAMPLE['state'], rules_resource.state)
        self.assertEqual(EXAMPLE['id'], rules_resource.id)
        self.assertEqual(EXAMPLE['access_type'], rules_resource.access_type)
        self.assertEqual(EXAMPLE['access_to'], rules_resource.access_to)
        self.assertEqual(EXAMPLE['access_key'], rules_resource.access_key)
        self.assertEqual(EXAMPLE['created_at'], rules_resource.created_at)
        self.assertEqual(EXAMPLE['updated_at'], rules_resource.updated_at)
        self.assertEqual(EXAMPLE['metadata'], rules_resource.metadata)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class ResourceLock(resource.Resource):
    resource_key = "resource_lock"
    resources_key = "resource_locks"
    base_path = "/resource-locks"

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True
    allow_head = False

    _query_mapping = resource.QueryParameters(
        "project_id",
        "created_since",
        "created_before",
        "limit",
        "offset",
        "id",
        "resource_id",
        "resource_type",
        "resource_action",
        "user_id",
        "lock_context",
        "lock_reason",
        "lock_reason~",
        "sort_key",
        "sort_dir",
        "with_count",
        "all_projects",
    )
    # The resource was introduced in this microversion, so it is the minimum
    # version to use it. Openstacksdk currently doesn't allow to set
    # minimum microversions.
    _max_microversion = '2.81'

    #: Properties
    #: The date and time stamp when the resource was created within the
    #: services's database.
    created_at = resource.Body("created_at", type=str)
    #: The date and time stamp when the resource was last modified within the
    #: services's database.
    updated_at = resource.Body("updated_at", type=str)
    #: The ID of the user that owns the lock
    user_id = resource.Body("user_id", type=str)
    #: The ID of the project that owns the lock.
    project_id = resource.Body("project_id", type=str)
    #: The type of the resource that is locked, i.e.: share, access rule.
    resource_type = resource.Body("resource_type", type=str)
    #: The UUID of the resource that is locked.
    resource_id = resource.Body("resource_id", type=str)
    #: What action is currently locked, i.e.: deletion, visibility of fields.
    resource_action = resource.Body("resource_action", type=str)
    #: The reason specified while the lock was being placed.
    lock_reason = resource.Body("lock_reason", type=str)
    #: The context that placed the lock (user, admin or service).
    lock_context = resource.Body("lock_context", type=str)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from openstack.cloud import _utils
from openstack.cloud import openstackcloud
from openstack import exceptions


class CoeCloudMixin(openstackcloud._OpenStackCloudMixin):
    def list_coe_clusters(self):
        """List COE (Container Orchestration Engine) cluster.

        :returns: A list of container infrastructure management ``Cluster``
            objects.
        :raises: :class:`~openstack.exceptions.SDKException` if something goes
            wrong during the OpenStack API call.
        """
        return list(self.container_infrastructure_management.clusters())

    def search_coe_clusters(self, name_or_id=None, filters=None):
        """Search COE cluster.

        :param name_or_id: cluster name or ID.
        :param filters: a dict containing additional filters to use.
        :param detail: a boolean to control if we need summarized or
            detailed output.

        :returns: A list of container infrastructure management ``Cluster``
            objects.
        :raises: :class:`~openstack.exceptions.SDKException` if something goes
            wrong during the OpenStack API call.
        """
        coe_clusters = self.list_coe_clusters()
        return _utils._filter_list(coe_clusters, name_or_id, filters)

    def get_coe_cluster(self, name_or_id, filters=None):
        """Get a COE cluster by name or ID.

        :param name_or_id: Name or ID of the cluster.
        :param filters:
            A dictionary of meta data to use for further filtering. Elements
            of this dictionary may, themselves, be dictionaries. Example::

                {'last_name': 'Smith', 'other': {'gender': 'Female'}}

            OR
            A string containing a jmespath expression for further filtering.
            Example:: "[?last_name==`Smith`] | [?other.gender]==`Female`]"

        :returns: A container infrastructure management ``Cluster`` object if
            found, else None.
        """
        return _utils._get_entity(self, 'coe_cluster', name_or_id, filters)

    def create_coe_cluster(
        self,
        name,
        cluster_template_id,
        **kwargs,
    ):
        """Create a COE cluster based on given cluster template.

        :param string name: Name of the cluster.
        :param string cluster_template_id: ID of the cluster template to use.
        :param dict kwargs: Any other arguments to pass in.

        :returns: The created container infrastructure management ``Cluster``
            object.
        :raises: :class:`~openstack.exceptions.SDKException` if something goes
            wrong during the OpenStack API call
        """
        cluster = self.container_infrastructure_management.create_cluster(
            name=name,
            cluster_template_id=cluster_template_id,
            **kwargs,
        )

        return cluster

    def delete_coe_cluster(self, name_or_id):
        """Delete a COE cluster.

        :param name_or_id: Name or unique ID of the cluster.

        :returns: True if the delete succeeded, False if the
            cluster was not found.
        :raises: :class:`~openstack.exceptions.SDKException` on operation
            error.
        """

        cluster = self.get_coe_cluster(name_or_id)

        if not cluster:
            self.log.debug(
                "COE Cluster %(name_or_id)s does not exist",
                {'name_or_id': name_or_id},
                exc_info=True,
            )
            return False

        self.container_infrastructure_management.delete_cluster(cluster)
        return True

    def update_coe_cluster(self, name_or_id, **kwargs):
        """Update a COE cluster.

        :param name_or_id: Name or ID of the COE cluster being updated.
        :param kwargs: Cluster attributes to be updated.

        :returns: The updated cluster ``Cluster`` object.

        :raises: :class:`~openstack.exceptions.SDKException` on operation
            error.
        """
        cluster = self.get_coe_cluster(name_or_id)
        if not cluster:
            raise exceptions.SDKException(
                f"COE cluster {name_or_id} not found."
            )

        cluster = self.container_infrastructure_management.update_cluster(
            cluster, **kwargs
        )

        return cluster

    def get_coe_cluster_certificate(self, cluster_id):
        """Get details about the CA certificate for a cluster by name or ID.

        :param cluster_id: ID of the cluster.

        :returns: Details about the CA certificate for the given cluster.
        """
        return (
            self.container_infrastructure_management.get_cluster_certificate(
                cluster_id
            )
        )

    def sign_coe_cluster_certificate(self, cluster_id, csr):
        """Sign client key and generate the CA certificate for a cluster

        :param cluster_id: UUID of the cluster.
        :param csr: Certificate Signing Request (CSR) for authenticating
            client key.The CSR will be used by Magnum to generate a signed
            certificate that client will use to communicate with the cluster.

        :returns: a dict representing the signed certs.
        :raises: :class:`~openstack.exceptions.SDKException` on operation
            error.
        """
        return self.container_infrastructure_management.create_cluster_certificate(  # noqa: E501
            cluster_uuid=cluster_id, csr=csr
        )

    def list_cluster_templates(self, detail=False):
        """List cluster templates.

        :param bool detail. Ignored. Included for backwards compat.
            ClusterTemplates are always returned with full details.

        :returns: a list of dicts containing the cluster template details.
        :raises: :class:`~openstack.exceptions.SDKException` if something goes
            wrong during the OpenStack API call.
        """
        return list(
            self.container_infrastructure_management.cluster_templates()
        )

    def search_cluster_templates(
        self, name_or_id=None, filters=None, detail=False
    ):
        """Search cluster templates.

        :param name_or_id: cluster template name or ID.
        :param filters: a dict containing additional filters to use.
        :param detail: a boolean to control if we need summarized or
            detailed output.

        :returns: a list of dict containing the cluster templates
        :raises: :class:`~openstack.exceptions.SDKException`: if something goes
            wrong during the OpenStack API call.
        """
        cluster_templates = self.list_cluster_templates(detail=detail)
        return _utils._filter_list(cluster_templates, name_or_id, filters)

    def get_cluster_template(self, name_or_id, filters=None, detail=False):
        """Get a cluster template by name or ID.

        :param name_or_id: Name or ID of the cluster template.
        :param filters:
            A dictionary of meta data to use for further filtering. Elements
            of this dictionary may, themselves, be dictionaries. Example::

                {'last_name': 'Smith', 'other': {'gender': 'Female'}}

            OR
            A string containing a jmespath expression for further filtering.
            Example:: "[?last_name==`Smith`] | [?other.gender]==`Female`]"

        :returns: A cluster template dict or None if no matching
            cluster template is found.
        """
        return _utils._get_entity(
            self,
            'cluster_template',
            name_or_id,
            filters=filters,
            detail=detail,
        )

    def create_cluster_template(
        self, name, image_id=None, keypair_id=None, coe=None, **kwargs
    ):
        """Create a cluster template.

        :param string name: Name of the cluster template.
        :param string image_id: Name or ID of the image to use.
        :param string keypair_id: Name or ID of the keypair to use.
        :param string coe: Name of the coe for the cluster template.
            Other arguments will be passed in kwargs.

        :returns: a dict containing the cluster template description
        :raises: :class:`~openstack.exceptions.SDKException` if something goes
            wrong during the OpenStack API call
        """
        cluster_template = (
            self.container_infrastructure_management.create_cluster_template(
                name=name,
                image_id=image_id,
                keypair_id=keypair_id,
                coe=coe,
                **kwargs,
            )
        )

        return cluster_template

    def delete_cluster_template(self, name_or_id):
        """Delete a cluster template.

        :param name_or_id: Name or unique ID of the cluster template.

        :returns: True if the delete succeeded, False if the
            cluster template was not found.
        :raises: :class:`~openstack.exceptions.SDKException` on operation
            error.
        """

        cluster_template = self.get_cluster_template(name_or_id)

        if not cluster_template:
            self.log.debug(
                "Cluster template %(name_or_id)s does not exist",
                {'name_or_id': name_or_id},
                exc_info=True,
            )
            return False

        self.container_infrastructure_management.delete_cluster_template(
            cluster_template
        )
        return True

    def update_cluster_template(self, name_or_id, **kwargs):
        """Update a cluster template.

        :param name_or_id: Name or ID of the cluster template being updated.

        :returns: an update cluster template.
        :raises: :class:`~openstack.exceptions.SDKException` on operation
            error.
        """
        cluster_template = self.get_cluster_template(name_or_id)
        if not cluster_template:
            raise exceptions.SDKException(
                f"Cluster template {name_or_id} not found."
            )

        cluster_template = (
            self.container_infrastructure_management.update_cluster_template(
                cluster_template, **kwargs
            )
        )

        return cluster_template

    def list_magnum_services(self):
        """List all Magnum services.

        :returns: a list of dicts containing the service details.
        :raises: :class:`~openstack.exceptions.SDKException` on operation
            error.
        """
        return list(self.container_infrastructure_management.services())



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.network.v2 import port
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'admin_state_up': True,
    'allowed_address_pairs': [{'2': 2}],
    'binding:host_id': '3',
    'binding:profile': {'4': 4},
    'binding:vif_details': {'5': 5},
    'binding:vif_type': '6',
    'binding:vnic_type': '7',
    'created_at': '2016-03-09T12:14:57.233772',
    'data_plane_status': '32',
    'description': '8',
    'device_id': '9',
    'device_owner': '10',
    'device_profile': 'cyborg_device_profile_1',
    'dns_assignment': [{'11': 11}],
    'dns_domain': 'a11',
    'dns_name': '12',
    'extra_dhcp_opts': [{'13': 13}],
    'fixed_ips': [{'14': '14'}],
    'hardware_offload_type': None,
    'id': IDENTIFIER,
    'ip_allocation': 'immediate',
    'mac_address': '16',
    'name': '17',
    'network_id': '18',
    'numa_affinity_policy': False,
    'port_security_enabled': True,
    'qos_network_policy_id': '32',
    'qos_policy_id': '21',
    'propagate_uplink_status': False,
    'resource_request': {
        'required': ['CUSTOM_PHYSNET_PUBLIC', 'CUSTOM_VNIC_TYPE_NORMAL'],
        'resources': {
            'NET_BW_EGR_KILOBIT_PER_SEC': 1,
            'NET_BW_IGR_KILOBIT_PER_SEC': 2,
        },
    },
    'revision_number': 22,
    'security_groups': ['23'],
    'status': '25',
    'project_id': '26',
    'trunk_details': {
        'trunk_id': '27',
        'sub_ports': [
            {
                'port_id': '28',
                'segmentation_id': 29,
                'segmentation_type': '30',
                'mac_address': '31',
            }
        ],
    },
    'trusted': True,
    'updated_at': '2016-07-09T12:14:57.233772',
}


class TestPort(base.TestCase):
    def test_basic(self):
        sot = port.Port()
        self.assertEqual('port', sot.resource_key)
        self.assertEqual('ports', sot.resources_key)
        self.assertEqual('/ports', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

        self.assertDictEqual(
            {
                "binding:host_id": "binding:host_id",
                "binding:profile": "binding:profile",
                "binding:vif_details": "binding:vif_details",
                "binding:vif_type": "binding:vif_type",
                "binding:vnic_type": "binding:vnic_type",
                "description": "description",
                "device_id": "device_id",
                "device_owner": "device_owner",
                "fields": "fields",
                "fixed_ips": "fixed_ips",
                "id": "id",
                "ip_address": "ip_address",
                "mac_address": "mac_address",
                "name": "name",
                "network_id": "network_id",
                "security_groups": "security_groups",
                "status": "status",
                "subnet_id": "subnet_id",
                "is_admin_state_up": "admin_state_up",
                "is_port_security_enabled": "port_security_enabled",
                "project_id": "project_id",
                "tenant_id": "project_id",
                "security_group_ids": "security_groups",
                "limit": "limit",
                "marker": "marker",
                "any_tags": "tags-any",
                "not_any_tags": "not-tags-any",
                "not_tags": "not-tags",
                "tags": "tags",
                'sort_dir': 'sort_dir',
                'sort_key': 'sort_key',
            },
            sot._query_mapping._mapping,
        )

    def test_make_it(self):
        sot = port.Port(**EXAMPLE)
        self.assertTrue(sot.is_admin_state_up)
        self.assertEqual(
            EXAMPLE['allowed_address_pairs'], sot.allowed_address_pairs
        )
        self.assertEqual(EXAMPLE['binding:host_id'], sot.binding_host_id)
        self.assertEqual(EXAMPLE['binding:profile'], sot.binding_profile)
        self.assertEqual(
            EXAMPLE['binding:vif_details'], sot.binding_vif_details
        )
        self.assertEqual(EXAMPLE['binding:vif_type'], sot.binding_vif_type)
        self.assertEqual(EXAMPLE['binding:vnic_type'], sot.binding_vnic_type)
        self.assertEqual(EXAMPLE['created_at'], sot.created_at)
        self.assertEqual(EXAMPLE['data_plane_status'], sot.data_plane_status)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['device_id'], sot.device_id)
        self.assertEqual(EXAMPLE['device_owner'], sot.device_owner)
        self.assertEqual(EXAMPLE['device_profile'], sot.device_profile)
        self.assertEqual(EXAMPLE['dns_assignment'], sot.dns_assignment)
        self.assertEqual(EXAMPLE['dns_domain'], sot.dns_domain)
        self.assertEqual(EXAMPLE['dns_name'], sot.dns_name)
        self.assertEqual(EXAMPLE['extra_dhcp_opts'], sot.extra_dhcp_opts)
        self.assertEqual(EXAMPLE['fixed_ips'], sot.fixed_ips)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['ip_allocation'], sot.ip_allocation)
        self.assertEqual(EXAMPLE['mac_address'], sot.mac_address)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['network_id'], sot.network_id)
        self.assertEqual(
            EXAMPLE['numa_affinity_policy'], sot.numa_affinity_policy
        )
        self.assertTrue(sot.is_port_security_enabled)
        self.assertEqual(
            EXAMPLE['qos_network_policy_id'], sot.qos_network_policy_id
        )
        self.assertEqual(EXAMPLE['qos_policy_id'], sot.qos_policy_id)
        self.assertEqual(
            EXAMPLE['propagate_uplink_status'], sot.propagate_uplink_status
        )
        self.assertEqual(EXAMPLE['resource_request'], sot.resource_request)
        self.assertEqual(EXAMPLE['revision_number'], sot.revision_number)
        self.assertEqual(EXAMPLE['security_groups'], sot.security_group_ids)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE['trunk_details'], sot.trunk_details)
        self.assertEqual(EXAMPLE['trusted'], sot.trusted)
        self.assertEqual(EXAMPLE['updated_at'], sot.updated_at)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import typing as ty

from keystoneauth1 import adapter
import typing_extensions as ty_ext

from openstack.dns.v2 import _base
from openstack import resource


class Quota(_base.Resource):
    """DNS Quota Resource"""

    base_path = "/quotas"

    # capabilities
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True
    commit_method = "PATCH"

    # Properties
    #: The ID of the project.
    project = resource.URI("project", alternate_id=True)
    #: The maximum amount of recordsets allowed in a zone export. *Type: int*
    api_export_size = resource.Body("api_export_size", type=int)
    #: The maximum amount of records allowed per recordset. *Type: int*
    recordset_records = resource.Body("recordset_records", type=int)
    #: The maximum amount of records allowed per zone. *Type: int*
    zone_records = resource.Body("zone_records", type=int)
    #: The maximum amount of recordsets allowed per zone. *Type: int*
    zone_recordsets = resource.Body("zone_recordsets", type=int)
    #: The maximum amount of zones allowed per project. *Type: int*
    zones = resource.Body("zones", type=int)

    def _prepare_request(
        self,
        requires_id=True,
        prepend_key=False,
        patch=False,
        base_path=None,
        params=None,
        *,
        resource_request_key=None,
        **kwargs,
    ):
        _request = super()._prepare_request(
            requires_id, prepend_key, base_path=base_path
        )
        if self.resource_key in _request.body:
            _body = _request.body[self.resource_key]
        else:
            _body = _request.body
        if "id" in _body:
            del _body["id"]
        _request.headers = {'x-auth-sudo-project-id': self.id}
        return _request

    def fetch(
        self,
        session: adapter.Adapter,
        requires_id: bool = True,
        base_path: str | None = None,
        error_message: str | None = None,
        skip_cache: bool = False,
        *,
        resource_response_key: str | None = None,
        microversion: str | None = None,
        **params: ty.Any,
    ) -> ty_ext.Self:
        request = self._prepare_request(
            requires_id=requires_id,
            base_path=base_path,
        )
        session = self._get_session(session)
        if microversion is None:
            microversion = self._get_microversion(session)
        self.microversion = microversion

        response = session.get(
            request.url,
            microversion=microversion,
            params=params,
            skip_cache=skip_cache,
            headers=request.headers,
        )

        self._translate_response(
            response,
            error_message=error_message,
            resource_response_key=resource_response_key,
        )

        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from openstack.tests.unit import base


IDENTIFIER = str(uuid.uuid4())
MANILA_AZ_DICT = {
    "id": IDENTIFIER,
    "name": "manila-zone-0",
    "created_at": "2021-01-21T20:13:55.000000",
    "updated_at": None,
}


class TestSharedFileSystem(base.TestCase):
    def setUp(self):
        super().setUp()
        self.use_manila()

    def test_list_availability_zones(self):
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'shared-file-system',
                        'public',
                        append=['v2', 'availability-zones'],
                    ),
                    json={'availability_zones': [MANILA_AZ_DICT]},
                ),
            ]
        )
        az_list = self.cloud.list_share_availability_zones()
        self.assertEqual(len(az_list), 1)
        self.assertEqual(MANILA_AZ_DICT['id'], az_list[0].id)
        self.assertEqual(MANILA_AZ_DICT['name'], az_list[0].name)
        self.assertEqual(MANILA_AZ_DICT['created_at'], az_list[0].created_at)
        self.assertEqual(MANILA_AZ_DICT['updated_at'], az_list[0].updated_at)
        self.assert_calls()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from keystoneauth1 import adapter

from openstack.dns.v2 import zone
from openstack.tests.unit import base

IDENTIFIER = 'NAME'
EXAMPLE = {
    'attributes': {'tier': 'gold', 'ha': 'true'},
    'id': IDENTIFIER,
    'name': 'test.org',
    'email': 'joe@example.org',
    'type': 'PRIMARY',
    'ttl': 7200,
    'description': 'This is an example zone.',
    'status': 'ACTIVE',
    'shared': False,
}


class TestZone(base.TestCase):
    def setUp(self):
        super().setUp()
        self.resp = mock.Mock()
        self.resp.body = None
        self.resp.json = mock.Mock(return_value=self.resp.body)
        self.resp.status_code = 200
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.post = mock.Mock(return_value=self.resp)
        self.sess.default_microversion = None

    def test_basic(self):
        sot = zone.Zone()
        self.assertEqual(None, sot.resource_key)
        self.assertEqual('zones', sot.resources_key)
        self.assertEqual('/zones', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

        self.assertEqual('PATCH', sot.commit_method)

        self.assertDictEqual(
            {
                'description': 'description',
                'email': 'email',
                'limit': 'limit',
                'marker': 'marker',
                'name': 'name',
                'status': 'status',
                'ttl': 'ttl',
                'type': 'type',
            },
            sot._query_mapping._mapping,
        )

    def test_make_it(self):
        sot = zone.Zone(**EXAMPLE)
        self.assertEqual(IDENTIFIER, sot.id)
        self.assertEqual(EXAMPLE['email'], sot.email)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['ttl'], sot.ttl)
        self.assertEqual(EXAMPLE['type'], sot.type)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['shared'], sot.is_shared)

    def test_abandon(self):
        sot = zone.Zone(**EXAMPLE)
        self.assertIsNone(sot.abandon(self.sess))
        self.sess.post.assert_called_with(
            'zones/NAME/tasks/abandon', json=None
        )



# Copyright (c) 2016 IBM
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
test_project
----------------------------------

Functional tests for project resource.
"""

import pprint

from openstack import exceptions
from openstack.tests.functional import base


class TestProject(base.KeystoneBaseFunctionalTest):
    def setUp(self):
        super().setUp()

        if not self.operator_cloud:
            self.skipTest("Operator cloud is required for this test")

        self.new_project_name = self.getUniqueString('project')
        self.addCleanup(self._cleanup_projects)

    def _cleanup_projects(self):
        exception_list = list()
        for p in self.operator_cloud.list_projects():
            if p['name'].startswith(self.new_project_name):
                try:
                    self.operator_cloud.delete_project(p['id'])
                except Exception as e:
                    exception_list.append(str(e))
                    continue
        if exception_list:
            raise exceptions.SDKException('\n'.join(exception_list))

    def test_create_project(self):
        project_name = self.new_project_name + '_create'

        params = {
            'name': project_name,
            'description': 'test_create_project',
            'domain_id': self.operator_cloud.get_domain('default')['id'],
        }

        project = self.operator_cloud.create_project(**params)

        self.assertIsNotNone(project)
        self.assertEqual(project_name, project['name'])
        self.assertEqual('test_create_project', project['description'])

        user_id = self.operator_cloud.current_user_id

        # Grant the current user access to the project
        self.assertTrue(
            self.operator_cloud.grant_role(
                'member', user=user_id, project=project['id'], wait=True
            )
        )
        self.addCleanup(
            self.operator_cloud.revoke_role,
            'member',
            user=user_id,
            project=project['id'],
            wait=True,
        )

        new_cloud = self.operator_cloud.connect_as_project(project)
        self.add_info_on_exception(
            'new_cloud_config', pprint.pformat(new_cloud.config.config)
        )
        location = new_cloud.current_location
        self.assertEqual(project_name, location['project']['name'])

    def test_update_project(self):
        project_name = self.new_project_name + '_update'

        params = {
            'name': project_name,
            'description': 'test_update_project',
            'enabled': True,
            'domain_id': self.operator_cloud.get_domain('default')['id'],
        }

        project = self.operator_cloud.create_project(**params)
        updated_project = self.operator_cloud.update_project(
            project_name, enabled=False, description='new'
        )
        self.assertIsNotNone(updated_project)
        self.assertEqual(project['id'], updated_project['id'])
        self.assertEqual(project['name'], updated_project['name'])
        self.assertEqual(updated_project['description'], 'new')
        self.assertTrue(project['enabled'])
        self.assertFalse(updated_project['enabled'])

        # Revert the description and verify the project is still disabled
        updated_project = self.operator_cloud.update_project(
            project_name, description=params['description']
        )
        self.assertIsNotNone(updated_project)
        self.assertEqual(project['id'], updated_project['id'])
        self.assertEqual(project['name'], updated_project['name'])
        self.assertEqual(
            project['description'], updated_project['description']
        )
        self.assertTrue(project['enabled'])
        self.assertFalse(updated_project['enabled'])

    def test_delete_project(self):
        project_name = self.new_project_name + '_delete'
        params = {
            'name': project_name,
            'domain_id': self.operator_cloud.get_domain('default')['id'],
        }
        project = self.operator_cloud.create_project(**params)
        self.assertIsNotNone(project)
        self.assertTrue(self.operator_cloud.delete_project(project['id']))

    def test_delete_project_not_found(self):
        self.assertFalse(self.operator_cloud.delete_project('doesNotExist'))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class Listener(resource.Resource):
    resource_key = 'listener'
    resources_key = 'listeners'
    base_path = '/lbaas/listeners'

    _allow_unknown_attrs_in_body = True

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'connection_limit',
        'default_pool_id',
        'default_tls_container_ref',
        'description',
        'name',
        'project_id',
        'protocol',
        'protocol_port',
        is_admin_state_up='admin_state_up',
    )

    # Properties
    #: The maximum number of connections permitted for this load balancer.
    #: Default is infinite.
    connection_limit = resource.Body('connection_limit')
    #: ID of default pool. Must have compatible protocol with listener.
    default_pool_id = resource.Body('default_pool_id')
    #: A reference to a container of TLS secrets.
    default_tls_container_ref = resource.Body('default_tls_container_ref')
    #: Description for the listener.
    description = resource.Body('description')
    #: The administrative state of the listener, which is up
    #: ``True`` or down ``False``. *Type: bool*
    is_admin_state_up = resource.Body('admin_state_up', type=bool)
    #: List of load balancers associated with this listener.
    #: *Type: list of dicts which contain the load balancer IDs*
    load_balancer_ids = resource.Body('loadbalancers')
    #: The ID of the load balancer associated with this listener.
    load_balancer_id = resource.Body('loadbalancer_id')
    #: Name of the listener
    name = resource.Body('name')
    #: The ID of the project this listener is associated with.
    project_id = resource.Body('project_id')
    #: The protocol of the listener, which is TCP, HTTP, HTTPS
    #: or TERMINATED_HTTPS.
    protocol = resource.Body('protocol')
    #: Port the listener will listen to, e.g. 80.
    protocol_port = resource.Body('protocol_port')
    #: A list of references to TLS secrets.
    #: *Type: list*
    sni_container_refs = resource.Body('sni_container_refs')



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import argparse
import sys
import typing as ty

from openstack.config.loader import OpenStackConfig

if ty.TYPE_CHECKING:
    from openstack.config import cloud_region


__all__ = [
    'OpenStackConfig',
    'cloud_region',
    'get_cloud_region',
]


# TODO(stephenfin): Expand kwargs once we've typed OpenstackConfig.get_one
def get_cloud_region(
    service_key: str | None = None,
    options: argparse.ArgumentParser | None = None,
    app_name: str | None = None,
    app_version: str | None = None,
    load_yaml_config: bool = True,
    load_envvars: bool = True,
    **kwargs: ty.Any,
) -> 'cloud_region.CloudRegion':
    """Retrieve a single CloudRegion and merge additional options

    :param service_key: Service this argparse should be specialized for, if
        known. This will be used as the default value for service_type.
    :param options: Parser to attach additional options to
    :param app_name: Name of the application to be added to User Agent.
    :param app_version: Version of the application to be added to User Agent.
    :param load_yaml_config: Whether to load configuration from clouds.yaml and
        related configuration files.
    :param load_envvars: Whether to load configuration from environment
        variables
    :returns: A populated
        :class:`~openstack.config.cloud.cloud_region.CloudRegion` object.
    """
    config = OpenStackConfig(
        load_yaml_config=load_yaml_config,
        load_envvars=load_envvars,
        app_name=app_name,
        app_version=app_version,
    )
    if options:
        service_keys = [service_key] if service_key is not None else []
        config.register_argparse_arguments(options, sys.argv, service_keys)
        parsed_options, _ = options.parse_known_args(sys.argv)
    else:
        parsed_options = None

    return config.get_one(argparse=parsed_options, **kwargs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.functional.image.v2 import base


class TestTask(base.BaseImageTest):
    def test_tasks(self):
        tasks = list(self.operator_cloud.image.tasks())
        # NOTE(stephenfin): Yes, this is a dumb test. Basically all that we're
        # checking is that the API endpoint is correct. It would be nice to
        # have a proper check here that includes creation of tasks but we don't
        # currently have the ability to do this and I'm not even sure if tasks
        # are still really a supported thing. A potential future work item,
        # perhaps.
        self.assertIsInstance(tasks, list)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from openstack.identity.v3 import user as _user
from openstack.tests.functional import base


class TestUser(base.BaseFunctionalTest):
    def setUp(self):
        super().setUp()

        self.username = self.getUniqueString('user')
        self.password = "test_password_123"
        self.email = f"{self.username}@example.com"
        self.description = "Test user for functional testing"

    def _delete_user(self, user):
        ret = self.operator_cloud.identity.delete_user(user)
        self.assertIsNone(ret)

    def test_user(self):
        # Create user
        user = self.operator_cloud.identity.create_user(
            name=self.username,
            password=self.password,
            email=self.email,
            description=self.description,
        )
        self.addCleanup(self._delete_user, user)
        self.assertIsInstance(user, _user.User)
        self.assertIsNotNone(user.id)
        self.assertEqual(self.username, user.name)
        self.assertEqual(self.email, user.email)
        self.assertEqual(self.description, user.description)

        # Update user
        new_email = f"updated_{self.username}@example.com"
        new_description = "Updated description for test user"

        updated_user = self.operator_cloud.identity.update_user(
            user.id, email=new_email, description=new_description
        )
        self.assertIsInstance(updated_user, _user.User)
        self.assertEqual(new_email, updated_user.email)
        self.assertEqual(new_description, updated_user.description)
        self.assertEqual(
            self.username, updated_user.name
        )  # Name should remain unchanged

        # Read user list
        users = list(self.operator_cloud.identity.users())
        self.assertIsInstance(users[0], _user.User)
        user_ids = {ep.id for ep in users}
        self.assertIn(user.id, user_ids)

        # Read user by ID
        user = self.operator_cloud.identity.get_user(user.id)
        self.assertIsInstance(user, _user.User)
        self.assertEqual(user.id, user.id)
        self.assertEqual(self.username, user.name)
        self.assertEqual(new_email, user.email)
        self.assertEqual(new_description, user.description)
